@@ -21,19 +21,21 @@
 import os
 
 from gi.repository import Gdk
-from gi.repository import Gtk
+from gi.repository import GLib
 from gi.repository import GObject
+from gi.repository import Gtk
 
 from pynicotine import slskmessages
-from pynicotine.gtkgui.utils import humanize
-from pynicotine.gtkgui.utils import human_speed
-from pynicotine.gtkgui.utils import initialise_columns
+from pynicotine.config import config
 from pynicotine.gtkgui.utils import load_ui_elements
-from pynicotine.gtkgui.utils import PopupMenu
-from pynicotine.gtkgui.utils import set_treeview_selected_row
-from pynicotine.gtkgui.utils import show_user_status_tooltip
 from pynicotine.gtkgui.utils import triggers_context_menu
-from pynicotine.gtkgui.utils import update_widget_visuals
+from pynicotine.gtkgui.widgets.popupmenu import PopupMenu
+from pynicotine.gtkgui.widgets.treeview import initialise_columns
+from pynicotine.gtkgui.widgets.treeview import set_treeview_selected_row
+from pynicotine.gtkgui.widgets.treeview import show_user_status_tooltip
+from pynicotine.gtkgui.widgets.theme import update_widget_visuals
+from pynicotine.utils import humanize
+from pynicotine.utils import human_speed
 
 
 class Interests:
@@ -50,6 +52,7 @@ class Interests:
         self.likes_model = Gtk.ListStore(str)
         self.likes_model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
 
+        self.likes_column_numbers = list(range(self.likes_model.get_n_columns()))
         cols = initialise_columns(
             None,
             self.LikesList,
@@ -59,19 +62,11 @@ class Interests:
         cols["i_like"].set_sort_column_id(0)
         self.LikesList.set_model(self.likes_model)
 
-        self.til_popup_menu = popup = PopupMenu(self.frame)
-
-        popup.setup(
-            ("#" + _("_Remove Item"), self.on_remove_thing_i_like),
-            ("#" + _("Re_commendations For Item"), self.on_recommend_item),
-            ("", None),
-            ("#" + _("_Search For Item"), self.on_recommend_search)
-        )
-
         self.dislikes = {}
         self.dislikes_model = Gtk.ListStore(str)
         self.dislikes_model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
 
+        self.dislikes_column_numbers = list(range(self.dislikes_model.get_n_columns()))
         cols = initialise_columns(
             None,
             self.DislikesList,
@@ -81,14 +76,13 @@ class Interests:
         cols["i_dislike"].set_sort_column_id(0)
         self.DislikesList.set_model(self.dislikes_model)
 
-        self.tidl_popup_menu = popup = PopupMenu(self.frame)
-
-        popup.setup(
-            ("#" + _("_Remove Item"), self.on_remove_thing_i_dislike),
-            ("", None),
-            ("#" + _("_Search For Item"), self.on_recommend_search)
+        self.recommendations_model = Gtk.ListStore(
+            str,  # (0) hrating
+            str,  # (1) item
+            int   # (2) rating
         )
 
+        self.recommendations_column_numbers = list(range(self.recommendations_model.get_n_columns()))
         cols = initialise_columns(
             None,
             self.RecommendationsList,
@@ -99,23 +93,15 @@ class Interests:
         cols["rating"].set_sort_column_id(2)
         cols["item"].set_sort_column_id(1)
 
-        self.recommendations_model = Gtk.ListStore(
+        self.RecommendationsList.set_model(self.recommendations_model)
+
+        self.unrecommendations_model = Gtk.ListStore(
             str,  # (0) hrating
             str,  # (1) item
             int   # (2) rating
         )
-        self.RecommendationsList.set_model(self.recommendations_model)
-
-        self.r_popup_menu = popup = PopupMenu(self.frame)
-
-        popup.setup(
-            ("$" + _("I _Like This"), self.on_like_recommendation),
-            ("$" + _("I _Dislike This"), self.on_dislike_recommendation),
-            ("#" + _("_Recommendations For Item"), self.on_recommend_recommendation),
-            ("", None),
-            ("#" + _("_Search For Item"), self.on_recommend_search)
-        )
 
+        self.unrecommendations_column_numbers = list(range(self.unrecommendations_model.get_n_columns()))
         cols = initialise_columns(
             None,
             self.UnrecommendationsList,
@@ -126,23 +112,20 @@ class Interests:
         cols["rating"].set_sort_column_id(2)
         cols["item"].set_sort_column_id(1)
 
-        self.unrecommendations_model = Gtk.ListStore(
-            str,  # (0) hrating
-            str,  # (1) item
-            int   # (2) rating
-        )
         self.UnrecommendationsList.set_model(self.unrecommendations_model)
 
-        self.ur_popup_menu = popup = PopupMenu(self.frame)
-
-        popup.setup(
-            ("$" + _("I _Like This"), self.on_like_recommendation),
-            ("$" + _("I _Dislike This"), self.on_dislike_recommendation),
-            ("#" + _("_Recommendations For Item"), self.on_recommend_recommendation),
-            ("", None),
-            ("#" + _("_Search For Item"), self.on_recommend_search)
+        self.recommendation_users = {}
+        self.recommendation_users_model = Gtk.ListStore(
+            GObject.TYPE_OBJECT,  # (0) status icon
+            str,                  # (1) user
+            str,                  # (2) hspeed
+            str,                  # (3) hfiles
+            int,                  # (4) status
+            GObject.TYPE_UINT64,  # (5) speed
+            GObject.TYPE_UINT64   # (6) file count
         )
 
+        self.recommendation_users_column_numbers = list(range(self.recommendation_users_model.get_n_columns()))
         cols = initialise_columns(
             None,
             self.RecommendationUsersList,
@@ -159,152 +142,189 @@ class Interests:
 
         cols["status"].get_widget().hide()
 
-        self.recommendation_users = {}
-        self.recommendation_users_model = Gtk.ListStore(
-            GObject.TYPE_OBJECT,  # (0) status icon
-            str,                  # (1) user
-            str,                  # (2) hspeed
-            str,                  # (3) hfiles
-            int,                  # (4) status
-            GObject.TYPE_UINT64,  # (5) speed
-            GObject.TYPE_UINT64   # (6) file count
-        )
         self.RecommendationUsersList.set_model(self.recommendation_users_model)
         self.recommendation_users_model.set_sort_column_id(1, Gtk.SortType.ASCENDING)
 
+        for thing in config.sections["interests"]["likes"]:
+            if thing and isinstance(thing, str):
+                self.likes[thing] = self.likes_model.insert_with_valuesv(-1, self.likes_column_numbers, [thing])
+
+        for thing in config.sections["interests"]["dislikes"]:
+            if thing and isinstance(thing, str):
+                self.dislikes[thing] = self.dislikes_model.insert_with_valuesv(-1, self.dislikes_column_numbers, [thing])
+
+        """ Popup """
+
+        self.til_popup_menu = popup = PopupMenu(self.frame)
+        popup.setup(
+            ("#" + _("_Remove Item"), self.on_remove_thing_i_like),
+            ("#" + _("Re_commendations For Item"), self.on_recommend_item),
+            ("", None),
+            ("#" + _("_Search For Item"), self.on_til_recommend_search)
+        )
+
+        self.tidl_popup_menu = popup = PopupMenu(self.frame)
+        popup.setup(
+            ("#" + _("_Remove Item"), self.on_remove_thing_i_dislike),
+            ("", None),
+            ("#" + _("_Search For Item"), self.on_tidl_recommend_search)
+        )
+
+        self.r_popup_menu = popup = PopupMenu(self.frame)
+        popup.setup(
+            ("$" + _("I _Like This"), self.on_like_recommendation),
+            ("$" + _("I _Dislike This"), self.on_dislike_recommendation),
+            ("#" + _("_Recommendations For Item"), self.on_recommend_recommendation),
+            ("", None),
+            ("#" + _("_Search For Item"), self.on_r_recommend_search)
+        )
+
         self.ru_popup_menu = popup = PopupMenu(self.frame)
         popup.setup_user_menu()
 
-        for thing in self.np.config.sections["interests"]["likes"]:
-            if thing and isinstance(thing, str):
-                self.likes[thing] = self.likes_model.append([thing])
-
-        for thing in self.np.config.sections["interests"]["dislikes"]:
-            if thing and isinstance(thing, str):
-                self.dislikes[thing] = self.dislikes_model.append([thing])
-
         self.update_visuals()
 
-    def on_tooltip(self, widget, x, y, keyboard_mode, tooltip):
-        return show_user_status_tooltip(widget, x, y, tooltip, 4)
+    def recommend_search(self, item):
+        self.frame.SearchEntry.set_text(item)
+        self.frame.change_main_page("search")
 
     def on_add_thing_i_like(self, widget, *args):
+
         thing = widget.get_text()
         widget.set_text("")
 
-        if thing and thing.lower() not in self.np.config.sections["interests"]["likes"]:
+        if thing and thing.lower() not in config.sections["interests"]["likes"]:
             thing = thing.lower()
-            self.np.config.sections["interests"]["likes"].append(thing)
-            self.likes[thing] = self.likes_model.append([thing])
-            self.np.config.write_configuration()
+            config.sections["interests"]["likes"].append(thing)
+            self.likes[thing] = self.likes_model.insert_with_valuesv(-1, self.likes_column_numbers, [thing])
+            config.write_configuration()
             self.np.queue.append(slskmessages.AddThingILike(thing))
 
     def on_add_thing_i_dislike(self, widget, *args):
+
         thing = widget.get_text()
         widget.set_text("")
 
-        if thing and thing.lower() not in self.np.config.sections["interests"]["dislikes"]:
+        if thing and thing.lower() not in config.sections["interests"]["dislikes"]:
             thing = thing.lower()
-            self.np.config.sections["interests"]["dislikes"].append(thing)
-            self.dislikes[thing] = self.dislikes_model.append([thing])
-            self.np.config.write_configuration()
+            config.sections["interests"]["dislikes"].append(thing)
+            self.dislikes[thing] = self.dislikes_model.insert_with_valuesv(-1, self.dislikes_column_numbers, [thing])
+            config.write_configuration()
             self.np.queue.append(slskmessages.AddThingIHate(thing))
 
-    def on_remove_thing_i_like(self, widget):
+    def on_remove_thing_i_like(self, *args):
+
         thing = self.til_popup_menu.get_user()
 
-        if thing not in self.np.config.sections["interests"]["likes"]:
+        if thing not in config.sections["interests"]["likes"]:
             return
 
         self.likes_model.remove(self.likes[thing])
         del self.likes[thing]
-        self.np.config.sections["interests"]["likes"].remove(thing)
+        config.sections["interests"]["likes"].remove(thing)
 
-        self.np.config.write_configuration()
+        config.write_configuration()
         self.np.queue.append(slskmessages.RemoveThingILike(thing))
 
-    def on_remove_thing_i_dislike(self, widget):
+    def on_til_recommend_search(self, *args):
+        self.recommend_search(self.til_popup_menu.get_user())
+
+    def on_remove_thing_i_dislike(self, *args):
+
         thing = self.tidl_popup_menu.get_user()
 
-        if thing not in self.np.config.sections["interests"]["dislikes"]:
+        if thing not in config.sections["interests"]["dislikes"]:
             return
 
         self.dislikes_model.remove(self.dislikes[thing])
         del self.dislikes[thing]
-        self.np.config.sections["interests"]["dislikes"].remove(thing)
+        config.sections["interests"]["dislikes"].remove(thing)
 
-        self.np.config.write_configuration()
+        config.write_configuration()
         self.np.queue.append(slskmessages.RemoveThingIHate(thing))
 
-    def on_like_recommendation(self, widget):
-        thing = widget.get_parent().get_user()
+    def on_tidl_recommend_search(self, *args):
+        self.recommend_search(self.tidl_popup_menu.get_user())
 
-        if widget.get_active() and \
-                thing and thing not in self.np.config.sections["interests"]["likes"]:
-            self.np.config.sections["interests"]["likes"].append(thing)
-            self.likes[thing] = self.likes_model.append([thing])
+    def on_like_recommendation(self, action, state, thing=None):
 
-            self.np.config.write_configuration()
+        if thing is None:
+            thing = self.r_popup_menu.get_user()
+
+        if state.get_boolean() and \
+                thing and thing not in config.sections["interests"]["likes"]:
+            config.sections["interests"]["likes"].append(thing)
+            self.likes[thing] = self.likes_model.insert_with_valuesv(-1, self.likes_column_numbers, [thing])
+
+            config.write_configuration()
             self.np.queue.append(slskmessages.AddThingILike(thing))
 
-        elif not widget.get_active() and \
-                thing and thing in self.np.config.sections["interests"]["likes"]:
+        elif not state and \
+                thing and thing in config.sections["interests"]["likes"]:
             self.likes_model.remove(self.likes[thing])
             del self.likes[thing]
-            self.np.config.sections["interests"]["likes"].remove(thing)
+            config.sections["interests"]["likes"].remove(thing)
 
-            self.np.config.write_configuration()
+            config.write_configuration()
             self.np.queue.append(slskmessages.RemoveThingILike(thing))
 
-    def on_dislike_recommendation(self, widget):
-        thing = widget.get_parent().get_user()
+        action.set_state(state)
 
-        if widget.get_active() and \
-                thing and thing not in self.np.config.sections["interests"]["dislikes"]:
-            self.np.config.sections["interests"]["dislikes"].append(thing)
-            self.dislikes[thing] = self.dislikes_model.append([thing])
+    def on_dislike_recommendation(self, action, state, thing=None):
 
-            self.np.config.write_configuration()
+        if thing is None:
+            thing = self.r_popup_menu.get_user()
+
+        if state.get_boolean() and \
+                thing and thing not in config.sections["interests"]["dislikes"]:
+            config.sections["interests"]["dislikes"].append(thing)
+            self.dislikes[thing] = self.dislikes_model.insert_with_valuesv(-1, self.dislikes_column_numbers, [thing])
+
+            config.write_configuration()
             self.np.queue.append(slskmessages.AddThingIHate(thing))
 
-        elif not widget.get_active() and \
-                thing and thing in self.np.config.sections["interests"]["dislikes"]:
+        elif not state and \
+                thing and thing in config.sections["interests"]["dislikes"]:
             self.dislikes_model.remove(self.dislikes[thing])
             del self.dislikes[thing]
-            self.np.config.sections["interests"]["dislikes"].remove(thing)
+            config.sections["interests"]["dislikes"].remove(thing)
 
-            self.np.config.write_configuration()
+            config.write_configuration()
             self.np.queue.append(slskmessages.RemoveThingIHate(thing))
 
-    def on_recommend_item(self, widget):
+        action.set_state(state)
+
+    def on_recommend_item(self, *args):
+
         thing = self.til_popup_menu.get_user()
         self.np.queue.append(slskmessages.ItemRecommendations(thing))
         self.np.queue.append(slskmessages.ItemSimilarUsers(thing))
 
-    def on_recommend_recommendation(self, widget):
+    def on_recommend_recommendation(self, *args):
+
         thing = self.r_popup_menu.get_user()
         self.np.queue.append(slskmessages.ItemRecommendations(thing))
         self.np.queue.append(slskmessages.ItemSimilarUsers(thing))
 
-    def on_recommend_search(self, widget):
-        thing = widget.get_parent().get_user()
-        self.frame.SearchEntry.set_text(thing)
-        self.frame.change_main_page("search")
+    def on_r_recommend_search(self, *args):
+        self.recommend_search(self.r_popup_menu.get_user())
 
-    def on_global_recommendations_clicked(self, widget):
+    def on_global_recommendations_clicked(self, *args):
         self.np.queue.append(slskmessages.GlobalRecommendations())
 
-    def on_recommendations_clicked(self, widget):
+    def on_recommendations_clicked(self, *args):
         self.np.queue.append(slskmessages.Recommendations())
 
-    def on_similar_users_clicked(self, widget):
+    def on_similar_users_clicked(self, *args):
         self.np.queue.append(slskmessages.SimilarUsers())
 
     def set_recommendations(self, title, recom):
         self.recommendations_model.clear()
 
         for (thing, rating) in recom.items():
-            self.recommendations_model.append([humanize(rating), thing, rating])
+            self.recommendations_model.insert_with_valuesv(
+                -1, self.recommendations_column_numbers, [humanize(rating), thing, rating]
+            )
 
         self.recommendations_model.set_sort_column_id(2, Gtk.SortType.DESCENDING)
 
@@ -312,7 +332,9 @@ class Interests:
         self.unrecommendations_model.clear()
 
         for (thing, rating) in recom.items():
-            self.unrecommendations_model.append([humanize(rating), thing, rating])
+            self.unrecommendations_model.insert_with_valuesv(
+                -1, self.unrecommendations_column_numbers, [humanize(rating), thing, rating]
+            )
 
         self.unrecommendations_model.set_sort_column_id(2, Gtk.SortType.ASCENDING)
 
@@ -333,7 +355,9 @@ class Interests:
         self.recommendation_users = {}
 
         for user in msg.users:
-            iterator = self.recommendation_users_model.append([self.frame.images["offline"], user, "0", "0", 0, 0, 0])
+            iterator = self.recommendation_users_model.insert_with_valuesv(
+                -1, self.recommendation_users_column_numbers, [GObject.Value(GObject.TYPE_OBJECT, self.frame.images["offline"]), user, "0", "0", 0, 0, 0]
+            )
             self.recommendation_users[user] = iterator
 
             # Request user status, speed and number of shared files
@@ -361,6 +385,83 @@ class Interests:
 
         return model.get_value(iterator, column)
 
+    def on_popup_til_menu(self, widget):
+
+        item = self.get_selected_item(widget, column=0)
+        if item is None:
+            return False
+
+        self.til_popup_menu.set_user(item)
+
+        self.til_popup_menu.popup()
+        return True
+
+    def on_til_list_clicked(self, widget, event):
+
+        if triggers_context_menu(event):
+            set_treeview_selected_row(widget, event)
+            return self.on_popup_til_menu(widget)
+
+        return False
+
+    def on_popup_tidl_menu(self, widget):
+
+        item = self.get_selected_item(widget, column=0)
+        if item is None:
+            return False
+
+        self.tidl_popup_menu.set_user(item)
+
+        self.tidl_popup_menu.popup()
+        return True
+
+    def on_tidl_list_clicked(self, widget, event):
+
+        if triggers_context_menu(event):
+            set_treeview_selected_row(widget, event)
+            return self.on_popup_tidl_menu(widget)
+
+        return False
+
+    def on_popup_r_menu(self, widget):
+
+        item = self.get_selected_item(widget, column=1)
+        if item is None:
+            return False
+
+        self.r_popup_menu.set_user(item)
+
+        actions = self.r_popup_menu.get_actions()
+        actions[_("I _Like This")].set_state(
+            GLib.Variant.new_boolean(item in config.sections["interests"]["likes"])
+        )
+        actions[_("I _Dislike This")].set_state(
+            GLib.Variant.new_boolean(item in config.sections["interests"]["dislikes"])
+        )
+
+        self.r_popup_menu.popup()
+        return True
+
+    def on_r_list_clicked(self, widget, event):
+
+        if triggers_context_menu(event):
+            set_treeview_selected_row(widget, event)
+            return self.on_popup_r_menu(widget)
+
+        return False
+
+    def on_popup_ru_menu(self, widget):
+
+        user = self.get_selected_item(widget, column=1)
+        if user is None:
+            return False
+
+        self.ru_popup_menu.set_user(user)
+        self.ru_popup_menu.toggle_user_items()
+
+        self.ru_popup_menu.popup()
+        return True
+
     def on_ru_list_clicked(self, widget, event):
 
         if triggers_context_menu(event):
@@ -377,103 +478,10 @@ class Interests:
 
         return False
 
-    def on_popup_ru_menu(self, widget):
-
-        user = self.get_selected_item(widget, column=1)
-        if user is None:
-            return False
-
-        self.ru_popup_menu.set_user(user)
-        self.ru_popup_menu.toggle_user_items()
-
-        self.ru_popup_menu.popup()
-        return True
-
-    def on_til_list_clicked(self, widget, event):
-
-        if triggers_context_menu(event):
-            set_treeview_selected_row(widget, event)
-            return self.on_popup_til_menu(widget)
-
-        return False
-
-    def on_popup_til_menu(self, widget):
-
-        item = self.get_selected_item(widget, column=0)
-        if item is None:
-            return False
-
-        self.til_popup_menu.set_user(item)
-
-        self.til_popup_menu.popup()
-        return True
-
-    def on_tidl_list_clicked(self, widget, event):
-
-        if triggers_context_menu(event):
-            set_treeview_selected_row(widget, event)
-            return self.on_popup_tidl_menu(widget)
-
-        return False
-
-    def on_popup_tidl_menu(self, widget):
-
-        item = self.get_selected_item(widget, column=0)
-        if item is None:
-            return False
-
-        self.tidl_popup_menu.set_user(item)
-
-        self.tidl_popup_menu.popup()
-        return True
-
-    def on_r_list_clicked(self, widget, event):
-
-        if triggers_context_menu(event):
-            set_treeview_selected_row(widget, event)
-            return self.on_popup_r_menu(widget)
-
-        return False
-
-    def on_popup_r_menu(self, widget):
-
-        item = self.get_selected_item(widget, column=1)
-        if item is None:
-            return False
-
-        self.r_popup_menu.set_user(item)
-
-        items = self.r_popup_menu.get_items()
-        items[_("I _Like This")].set_active(item in self.np.config.sections["interests"]["likes"])
-        items[_("I _Dislike This")].set_active(item in self.np.config.sections["interests"]["dislikes"])
-
-        self.r_popup_menu.popup()
-        return True
-
-    def on_un_rec_list_clicked(self, widget, event):
-
-        if triggers_context_menu(event):
-            set_treeview_selected_row(widget, event)
-            return self.on_popup_un_rec_menu(widget)
-
-        return False
-
-    def on_popup_un_rec_menu(self, widget):
-
-        item = self.get_selected_item(widget, column=1)
-        if item is None:
-            return False
-
-        self.ur_popup_menu.set_user(item)
-
-        items = self.ur_popup_menu.get_items()
-        items[_("I _Like This")].set_active(item in self.np.config.sections["interests"]["likes"])
-        items[_("I _Dislike This")].set_active(item in self.np.config.sections["interests"]["dislikes"])
-
-        self.ur_popup_menu.popup()
-        return True
+    def on_tooltip(self, widget, x, y, keyboard_mode, tooltip):
+        return show_user_status_tooltip(widget, x, y, tooltip, 4)
 
     def update_visuals(self):
 
-        for widget in self.__dict__.values():
+        for widget in list(self.__dict__.values()):
             update_widget_visuals(widget)
