@@ -23,6 +23,7 @@ from gi.repository import Gdk
 from gi.repository import Gio
 from gi.repository import GLib
 
+from pynicotine.config import config
 from pynicotine.logfacility import log
 from pynicotine.utils import execute_command
 
@@ -37,12 +38,19 @@ class Notifications:
         self.tts_playing = False
         self.continue_playing = False
 
+        if sys.platform == "win32":
+            """ Windows notification popup support via plyer
+            Once https://gitlab.gnome.org/GNOME/glib/-/issues/1234 is implemented, we can drop plyer """
+
+            from plyer import notification
+            self.win_notification = notification
+
     def add(self, location, user, room=None):
 
         if location == "rooms" and room is not None and user is not None:
             if room not in self.frame.hilites[location]:
                 self.frame.hilites[location].append(room)
-                self.frame.tray.set_image()
+                self.frame.tray_icon.set_image()
 
         elif location == "private":
             if user in self.frame.hilites[location]:
@@ -51,9 +59,9 @@ class Notifications:
 
             elif user not in self.frame.hilites[location]:
                 self.frame.hilites[location].append(user)
-                self.frame.tray.set_image()
+                self.frame.tray_icon.set_image()
 
-        if self.frame.np.config.sections["ui"]["urgencyhint"] and \
+        if config.sections["ui"]["urgencyhint"] and \
                 not self.frame.MainWindow.is_active():
             self.frame.MainWindow.set_urgency_hint(True)
 
@@ -73,7 +81,7 @@ class Notifications:
 
             self.set_title(user)
 
-        self.frame.tray.set_image()
+        self.frame.tray_icon.set_image()
 
     def set_title(self, user=None):
 
@@ -84,7 +92,7 @@ class Notifications:
             self.frame.MainWindow.set_title(app_name)
             return
 
-        if not self.frame.np.config.sections["notifications"]["notification_window_title"]:
+        if not config.sections["notifications"]["notification_window_title"]:
             return
 
         if self.frame.hilites["private"]:
@@ -110,23 +118,26 @@ class Notifications:
 
     def new_tts(self, message):
 
-        if not self.frame.np.config.sections["ui"]["speechenabled"]:
+        if not config.sections["ui"]["speechenabled"]:
             return
 
-        if message not in self.tts:
-            self.tts.append(message)
+        if message in self.tts:
+            return
 
-            thread = threading.Thread(target=self.play_tts)
-            thread.name = "TTS"
-            thread.daemon = True
-            thread.start()
-
-    def play_tts(self):
+        self.tts.append(message)
 
         if self.tts_playing:
+            # Avoid spinning up useless threads
             self.continue_playing = True
             return
 
+        thread = threading.Thread(target=self.play_tts)
+        thread.name = "TTS"
+        thread.daemon = True
+        thread.start()
+
+    def play_tts(self):
+
         for message in self.tts[:]:
             self.tts_player(message)
 
@@ -149,7 +160,11 @@ class Notifications:
 
         self.tts_playing = True
 
-        execute_command(self.frame.np.config.sections["ui"]["speechcommand"], message)
+        try:
+            execute_command(config.sections["ui"]["speechcommand"], message, background=False)
+
+        except Exception as error:
+            log.add_warning(_("Text-to-speech for message failed: %s"), str(error))
 
     def new_notification(self, message, title=None, priority=Gio.NotificationPriority.NORMAL):
 
@@ -158,19 +173,13 @@ class Notifications:
 
         try:
             if sys.platform == "win32":
-
-                """ Windows notification popup support via plyer
-                Once https://gitlab.gnome.org/GNOME/glib/-/issues/1234 is implemented, we can drop plyer """
-
-                from plyer import notification
-
-                notification.notify(
+                self.win_notification.notify(
                     app_name=GLib.get_application_name(),
                     title=title,
-                    message=message
+                    message=message[:256]
                 )
 
-                if self.frame.np.config.sections["notifications"]["notification_popup_sound"]:
+                if config.sections["notifications"]["notification_popup_sound"]:
                     import winsound
                     winsound.PlaySound("SystemAsterisk", winsound.SND_ALIAS)
                 return
@@ -185,7 +194,7 @@ class Notifications:
 
             self.application.send_notification(None, notification_popup)
 
-            if self.frame.np.config.sections["notifications"]["notification_popup_sound"]:
+            if config.sections["notifications"]["notification_popup_sound"]:
                 Gdk.beep()
 
         except Exception as error:
