@@ -27,68 +27,60 @@ from collections import deque
 from time import altzone
 from time import daylight
 
-from gi.repository import Gdk
 from gi.repository import Gio
 from gi.repository import GLib
-from gi.repository import GObject
 from gi.repository import Gtk
 
 from pynicotine import slskmessages
-from pynicotine.gtkgui.chatrooms import get_completion
-from pynicotine.gtkgui.dialogs import option_dialog
-from pynicotine.gtkgui.utils import add_alias
+from pynicotine.config import config
 from pynicotine.gtkgui.utils import append_line
 from pynicotine.gtkgui.utils import auto_replace
 from pynicotine.gtkgui.utils import censor_chat
+from pynicotine.gtkgui.utils import copy_all_text
 from pynicotine.gtkgui.utils import delete_log
-from pynicotine.gtkgui.utils import entry_completion_find_match
-from pynicotine.gtkgui.utils import entry_completion_found_match
-from pynicotine.gtkgui.utils import expand_alias
-from pynicotine.gtkgui.utils import get_user_status_color
-from pynicotine.gtkgui.utils import IconNotebook
-from pynicotine.gtkgui.utils import is_alias
-from pynicotine.gtkgui.utils import keyval_to_hardware_keycode
 from pynicotine.gtkgui.utils import load_ui_elements
 from pynicotine.gtkgui.utils import open_log
-from pynicotine.gtkgui.utils import PopupMenu
 from pynicotine.gtkgui.utils import scroll_bottom
-from pynicotine.gtkgui.utils import TextSearchBar
 from pynicotine.gtkgui.utils import triggers_context_menu
-from pynicotine.gtkgui.utils import unalias
-from pynicotine.gtkgui.utils import update_tag_visuals
-from pynicotine.gtkgui.utils import update_widget_visuals
+from pynicotine.gtkgui.widgets.iconnotebook import IconNotebook
+from pynicotine.gtkgui.widgets.popupmenu import PopupMenu
+from pynicotine.gtkgui.widgets.messagedialogs import option_dialog
+from pynicotine.gtkgui.widgets.textentry import ChatEntry
+from pynicotine.gtkgui.widgets.textentry import TextSearchBar
+from pynicotine.gtkgui.widgets.theme import get_user_status_color
+from pynicotine.gtkgui.widgets.theme import update_tag_visuals
+from pynicotine.gtkgui.widgets.theme import update_widget_visuals
 from pynicotine.logfacility import log
+from pynicotine.utils import get_completion_list
 from pynicotine.utils import get_path
 from pynicotine.utils import version
 
 
-CTCP_VERSION = "\x01VERSION\x01"
-
-
 class PrivateChats(IconNotebook):
 
-    CMDS = set(
-        [
-            "/alias ", "/unalias ", "/whois ", "/browse ", "/ip ", "/pm ", "/msg ", "/search ", "/usearch ", "/rsearch ",
-            "/bsearch ", "/join ", "/add ", "/buddy ", "/rem ", "/unbuddy ", "/ban ", "/ignore ", "/ignoreip ", "/unban ", "/unignore ",
-            "/clear ", "/quit ", "/exit ", "/rescan ", "/info ", "/ctcpversion "
-        ]
-    )
+    # List of allowed commands. The implementation for them is in the ChatEntry class.
+    CMDS = {
+        "/al ", "/alias ", "/un ", "/unalias ", "/w ", "/whois ", "/browse ", "/b ", "/ip ", "/pm ", "/m ", "/msg ",
+        "/s ", "/search ", "/us ", "/usearch ", "/rs ", "/rsearch ", "/bs ", "/bsearch ", "/j ", "/join ",
+        "/ad ", "/add ", "/buddy ", "/rem ", "/unbuddy ", "/ban ", "/ignore ", "/ignoreip ", "/unban ",
+        "/unignore ", "/clear ", "/cl ", "/me ", "/a ", "/away ", "/q ", "/quit ", "/exit ", "/now ", "/rescan ",
+        "/info ", "/toggle ", "/ctcpversion "
+    }
+
+    CTCP_VERSION = "\x01VERSION\x01"
 
     def __init__(self, frame):
 
         self.frame = frame
 
-        config = frame.np.config.sections
-
         IconNotebook.__init__(
             self,
             self.frame.images,
-            angle=config["ui"]["labelprivate"],
-            tabclosers=config["ui"]["tabclosers"],
-            show_hilite_image=config["notifications"]["notification_tab_icons"],
-            reorderable=config["ui"]["tab_reorderable"],
-            show_status_image=config["ui"]["tab_status_icons"],
+            angle=config.sections["ui"]["labelprivate"],
+            tabclosers=config.sections["ui"]["tabclosers"],
+            show_hilite_image=config.sections["notifications"]["notification_tab_icons"],
+            reorderable=config.sections["ui"]["tab_reorderable"],
+            show_status_image=config.sections["ui"]["tab_status_icons"],
             notebookraw=self.frame.PrivatechatNotebookRaw
         )
 
@@ -96,14 +88,14 @@ class PrivateChats(IconNotebook):
 
         self.connected = True
         self.users = {}
-        self.clist = []
+        self.completion_list = []
         self.private_message_queue = {}
 
         self.notebook.connect("switch-page", self.on_switch_chat)
 
         # Clear list of previously open chats if we don't want to restore them
-        if not frame.np.config.sections["privatechat"]["store"]:
-            frame.np.config.sections["privatechat"]["users"].clear()
+        if not config.sections["privatechat"]["store"]:
+            config.sections["privatechat"]["users"].clear()
 
     def on_switch_chat(self, notebook, page, page_num, forceupdate=False):
 
@@ -147,11 +139,10 @@ class PrivateChats(IconNotebook):
                 # Offline
                 status = 0
 
-            tab = PrivateChat(self, user, status)
-            self.users[user] = tab
+            self.users[user] = tab = PrivateChat(self, user, status)
 
-            if user not in self.frame.np.config.sections["privatechat"]["users"]:
-                self.frame.np.config.sections["privatechat"]["users"].append(user)
+            if user not in config.sections["privatechat"]["users"]:
+                config.sections["privatechat"]["users"].append(user)
 
             # Get notified of user status
             self.frame.np.watch_user(user)
@@ -172,15 +163,8 @@ class PrivateChats(IconNotebook):
         if username not in self.users:
             return False
 
-        menu = PopupMenu(self.frame)
-        menu.setup_user_menu(username)
-        menu.get_items()[_("Send _Message")].set_visible(False)
-
-        menu.append_item(("", None))
-        menu.append_item(("#" + _("Close All Tabs"), self.users[username].on_close_all_tabs))
-        menu.append_item(("#" + _("_Close Tab"), self.users[username].on_close))
+        menu = self.users[username].popup_menu_user
         menu.toggle_user_items()
-
         menu.popup()
         return True
 
@@ -223,7 +207,7 @@ class PrivateChats(IconNotebook):
         # Update tray icon and show urgency hint
         self.frame.notifications.add("private", user)
 
-        if self.frame.np.config.sections["notifications"]["notification_popup_private_message"]:
+        if config.sections["notifications"]["notification_popup_private_message"]:
             self.frame.notifications.new_notification(
                 text,
                 title=_("Private message from %s") % user,
@@ -256,13 +240,13 @@ class PrivateChats(IconNotebook):
 
         # SEND CLIENT VERSION to user if the following string is sent
         ctcpversion = 0
-        if text == CTCP_VERSION:
+        if text == self.CTCP_VERSION:
             ctcpversion = 1
             text = "CTCP VERSION"
 
         self.users[msg.user].show_message(text, newmessage, msg.timestamp)
 
-        if ctcpversion and self.frame.np.config.sections["server"]["ctcpmsgs"] == 0:
+        if ctcpversion and config.sections["server"]["ctcpmsgs"] == 0:
             self.send_message(msg.user, GLib.get_application_name() + " " + version)
 
         self.frame.np.pluginhandler.incoming_private_chat_notification(msg.user, text)
@@ -279,11 +263,8 @@ class PrivateChats(IconNotebook):
 
         del self.users[tab.user]
 
-        # Update completions on exit
-        self.update_completions()
-
-        if tab.user in self.frame.np.config.sections["privatechat"]["users"]:
-            self.frame.np.config.sections["privatechat"]["users"].remove(tab.user)
+        if tab.user in config.sections["privatechat"]["users"]:
+            config.sections["privatechat"]["users"].remove(tab.user)
 
         self.remove_page(tab.Main)
 
@@ -297,10 +278,10 @@ class PrivateChats(IconNotebook):
             # Get notified of user status
             self.frame.np.watch_user(user)
 
-        if not self.frame.np.config.sections["privatechat"]["store"]:
+        if not config.sections["privatechat"]["store"]:
             return
 
-        for user in self.frame.np.config.sections["privatechat"]["users"]:
+        for user in config.sections["privatechat"]["users"]:
             if isinstance(user, str) and user not in self.users:
                 self.send_message(user)
 
@@ -316,22 +297,10 @@ class PrivateChats(IconNotebook):
 
     def update_completions(self):
 
-        config = self.frame.np.config.sections["words"]
-        clist = [self.frame.np.config.sections["server"]["login"], "nicotine"] + list(self.users.keys())
+        self.completion_list = get_completion_list(self.CMDS, self.frame.chatrooms.roomlist.server_rooms)
 
-        if config["buddies"]:
-            clist += [i[0] for i in self.frame.np.config.sections["server"]["userlist"]]
-
-        if config["aliases"]:
-            clist += ["/" + k for k in list(self.frame.np.config.sections["server"]["command_aliases"].keys())]
-
-        if config["commands"]:
-            clist += self.CMDS
-
-        self.clist = clist
-
-        for user in list(self.users.values()):
-            user.get_completion_list(clist=list(self.clist))
+        for user in self.users.values():
+            user.set_completion_list(list(self.completion_list))
 
 
 class PrivateChat:
@@ -342,49 +311,30 @@ class PrivateChat:
         self.chats = chats
         self.frame = chats.frame
 
-        # We should reference the user as soon as possible
-        self.chats.users[self.user] = self
-
         load_ui_elements(self, os.path.join(self.frame.gui_dir, "ui", "privatechat.ui"))
 
-        self.autoreplied = 0
-        self.offlinemessage = 0
+        self.autoreplied = False
+        self.offlinemessage = False
         self.status = status
-        self.clist = []
 
         # Text Search
         TextSearchBar(self.ChatScroll, self.SearchBar, self.SearchEntry)
 
-        # Spell Check
-        if self.frame.spell_checker is None:
-            self.frame.init_spell_checker()
+        # Chat Entry
+        self.entry = ChatEntry(self.frame, self.ChatLine, user, slskmessages.MessageUser, self.send_message, self.chats.CMDS, self.ChatScroll)
 
-        if self.frame.spell_checker and self.frame.np.config.sections["ui"]["spellcheck"]:
-            from gi.repository import Gspell
-            spell_buffer = Gspell.EntryBuffer.get_from_gtk_entry_buffer(self.ChatLine.get_buffer())
-            spell_buffer.set_spell_checker(self.frame.spell_checker)
-            spell_view = Gspell.Entry.get_from_gtk_entry(self.ChatLine)
-            spell_view.set_inline_spell_checking(True)
+        self.Log.set_active(config.sections["logging"]["privatechat"])
 
-        completion = Gtk.EntryCompletion()
-        self.ChatLine.set_completion(completion)
-        liststore = Gtk.ListStore(GObject.TYPE_STRING)
-        completion.set_model(liststore)
-
-        completion.set_text_column(0)
-        completion.set_match_func(entry_completion_find_match)
-        completion.connect("match-selected", entry_completion_found_match)
-
-        self.Log.set_active(self.frame.np.config.sections["logging"]["privatechat"])
-
-        self.popup_menu_user = popup = PopupMenu(self.frame, False)
-        popup.setup_user_menu(user)
-        popup.get_items()[_("Send _Message")].set_visible(False)
+        self.popup_menu_user = popup = PopupMenu(self.frame)
+        popup.setup_user_menu(user, page="privatechat")
+        popup.setup(
+            ("", None),
+            ("#" + _("Close All Tabs"), self.on_close_all_tabs),
+            ("#" + _("_Close Tab"), self.on_close)
+        )
 
         self.popup_menu = popup = PopupMenu(self.frame)
         popup.setup(
-            ("USERMENU", _("User"), self.popup_menu_user, self.on_popup_menu_user),
-            ("", None),
             ("#" + _("Find"), self.on_find_chat_log),
             ("", None),
             ("#" + _("Copy"), self.on_copy_chat_log),
@@ -394,31 +344,29 @@ class PrivateChat:
             ("#" + _("Delete Chat Log"), self.on_delete_chat_log),
             ("", None),
             ("#" + _("Clear Message View"), self.on_clear_messages),
-            ("#" + _("_Close Tab"), self.on_close)
+            ("", None),
+            (">" + _("User"), self.popup_menu_user),
         )
-
         popup.set_user(user)
 
         self.create_tags()
         self.update_visuals()
-
-        self.chats.update_completions()
+        self.set_completion_list(list(self.chats.completion_list))
 
         self.read_private_log()
 
     def read_private_log(self):
 
         # Read log file
-        config = self.frame.np.config.sections
         filename = self.user.replace(os.sep, "-") + ".log"
 
         try:
-            numlines = int(config["logging"]["readprivatelines"])
+            numlines = int(config.sections["logging"]["readprivatelines"])
         except Exception:
             numlines = 15
 
         try:
-            get_path(config["logging"]["privatelogsdir"], filename, self.append_log_lines, numlines)
+            get_path(config.sections["logging"]["privatelogsdir"], filename, self.append_log_lines, numlines)
 
         except IOError:
             pass
@@ -443,72 +391,63 @@ class PrivateChat:
                 append_line(self.ChatScroll, line, self.tag_hilite, timestamp_format="", username=self.user, usertag=self.tag_hilite, scroll=False)
 
     def login(self):
-        timestamp_format = self.frame.np.config.sections["logging"]["private_timestamp"]
+        timestamp_format = config.sections["logging"]["private_timestamp"]
         append_line(self.ChatScroll, _("--- reconnected ---"), self.tag_hilite, timestamp_format=timestamp_format)
         self.update_tags()
 
     def conn_close(self):
-        timestamp_format = self.frame.np.config.sections["logging"]["private_timestamp"]
+        timestamp_format = config.sections["logging"]["private_timestamp"]
         append_line(self.ChatScroll, _("--- disconnected ---"), self.tag_hilite, timestamp_format=timestamp_format)
         self.status = -1
-        self.offlinemessage = 0
+        self.offlinemessage = False
         self.update_tags()
 
     def on_message_view_clicked(self, widget, event):
 
         if triggers_context_menu(event):
-            return self.on_popup_menu(widget)
+            return self.on_popup_menu()
 
         return False
 
-    def on_popup_menu(self, widget):
+    def on_popup_menu(self, *args):
+
+        self.popup_menu_user.toggle_user_items()
         self.popup_menu.popup()
         return True
 
-    def on_popup_menu_user(self, widget):
-        self.popup_menu_user.toggle_user_items()
-        return True
-
-    def on_find_chat_log(self, widget):
+    def on_find_chat_log(self, *args):
         self.SearchBar.set_search_mode(True)
 
-    def on_copy_chat_log(self, widget):
+    def on_copy_chat_log(self, *args):
+        self.ChatScroll.emit("copy-clipboard")
 
-        bound = self.ChatScroll.get_buffer().get_selection_bounds()
-        if bound is not None and len(bound) == 2:
-            start, end = bound
-            log = self.ChatScroll.get_buffer().get_text(start, end, True)
-            self.frame.clip.set_text(log, -1)
+    def on_copy_all_chat_log(self, *args):
+        copy_all_text(self.ChatScroll, self.frame.clipboard)
 
-    def on_copy_all_chat_log(self, widget):
-        start, end = self.ChatScroll.get_buffer().get_bounds()
-        log = self.ChatScroll.get_buffer().get_text(start, end, True)
-        self.frame.clip.set_text(log, -1)
+    def on_view_chat_log(self, *args):
+        open_log(config.sections["logging"]["privatelogsdir"], self.user)
 
-    def on_view_chat_log(self, widget):
-        open_log(self.frame.np.config.sections["logging"]["privatelogsdir"], self.user)
-
-    def delete_chat_log_response(self, dialog, response, data):
-
-        if response == Gtk.ResponseType.OK:
-            delete_log(self.frame.np.config.sections["logging"]["privatelogsdir"], self.user)
-            self.on_clear_messages(dialog)
+    def on_delete_chat_log_response(self, dialog, response_id, data):
 
         dialog.destroy()
 
-    def on_delete_chat_log(self, widget):
+        if response_id == Gtk.ResponseType.OK:
+            delete_log(config.sections["logging"]["privatelogsdir"], self.user)
+            self.on_clear_messages()
+
+    def on_delete_chat_log(self, *args):
 
         option_dialog(
             parent=self.frame.MainWindow,
             title=_('Delete Logged Messages?'),
             message=_('Are you sure you wish to permanently delete all logged messages for this user?'),
-            callback=self.delete_chat_log_response
+            callback=self.on_delete_chat_log_response
         )
 
-    def on_clear_messages(self, widget):
+    def on_clear_messages(self, *args):
         self.ChatScroll.get_buffer().set_text("")
 
-    def on_show_chat_help(self, widget):
+    def on_show_chat_help(self, *args):
 
         if not hasattr(self, "AboutPrivateChatCommandsPopover"):
             load_ui_elements(self, os.path.join(self.frame.gui_dir, "ui", "popovers", "privatechatcommands.ui"))
@@ -533,7 +472,7 @@ class PrivateChat:
             speech = censor_chat(text)
             tag = self.tag_remote
 
-        timestamp_format = self.frame.np.config.sections["logging"]["private_timestamp"]
+        timestamp_format = config.sections["logging"]["private_timestamp"]
         if not newmessage and not self.offlinemessage:
             append_line(
                 self.ChatScroll,
@@ -541,7 +480,7 @@ class PrivateChat:
                 self.tag_hilite,
                 timestamp_format=timestamp_format
             )
-            self.offlinemessage = 1
+            self.offlinemessage = True
 
         if newmessage and self.offlinemessage:
             self.offlinemessage = False
@@ -560,16 +499,16 @@ class PrivateChat:
             append_line(self.ChatScroll, line, tag, timestamp_format=timestamp_format, username=self.user, usertag=self.tag_username)
 
         if self.Log.get_active():
-            timestamp_format = self.frame.np.config.sections["logging"]["log_timestamp"]
-            log.write_log(self.frame.np.config.sections["logging"]["privatelogsdir"], self.user, line, timestamp_format)
+            timestamp_format = config.sections["logging"]["log_timestamp"]
+            log.write_log(config.sections["logging"]["privatelogsdir"], self.user, line, timestamp_format)
 
-        autoreply = self.frame.np.config.sections["server"]["autoreply"]
+        autoreply = config.sections["server"]["autoreply"]
         if self.frame.away and not self.autoreplied and autoreply:
             self.send_message("[Auto-Message] %s" % autoreply)
-            self.autoreplied = 1
+            self.autoreplied = True
 
         self.frame.notifications.new_tts(
-            self.frame.np.config.sections["ui"]["speechprivate"] % {
+            config.sections["ui"]["speechprivate"] % {
                 "user": self.frame.notifications.tts_clean(self.user),
                 "message": self.frame.notifications.tts_clean(speech)
             }
@@ -577,20 +516,23 @@ class PrivateChat:
 
     def send_message(self, text, bytestring=False):
 
+        if not self.chats.connected:
+            return
+
         user_text = self.frame.np.pluginhandler.outgoing_private_chat_event(self.user, text)
         if user_text is None:
             return
 
         (u, text) = user_text
 
-        my_username = self.frame.np.config.sections["server"]["login"]
+        my_username = config.sections["server"]["login"]
 
         if text[:4] == "/me ":
             line = "* %s %s" % (my_username, text[4:])
             usertag = tag = self.tag_me
         else:
 
-            if text == CTCP_VERSION:
+            if text == self.chats.CTCP_VERSION:
                 line = "CTCP VERSION"
             else:
                 line = text
@@ -599,12 +541,12 @@ class PrivateChat:
             usertag = self.tag_my_username
             line = "[%s] %s" % (my_username, line)
 
-        timestamp_format = self.frame.np.config.sections["logging"]["private_timestamp"]
+        timestamp_format = config.sections["logging"]["private_timestamp"]
         append_line(self.ChatScroll, line, tag, timestamp_format=timestamp_format, username=my_username, usertag=usertag)
 
         if self.Log.get_active():
-            timestamp_format = self.frame.np.config.sections["logging"]["log_timestamp"]
-            log.write_log(self.frame.np.config.sections["logging"]["privatelogsdir"], self.user, line, timestamp_format)
+            timestamp_format = config.sections["logging"]["log_timestamp"]
+            log.write_log(config.sections["logging"]["privatelogsdir"], self.user, line, timestamp_format)
 
         if bytestring:
             payload = text
@@ -619,189 +561,9 @@ class PrivateChat:
 
         self.frame.np.pluginhandler.outgoing_private_chat_notification(self.user, text)
 
-    def on_enter(self, widget):
-
-        text = widget.get_text()
-
-        if not text:
-            widget.set_text("")
-            return
-
-        if is_alias(text):
-            new_text = expand_alias(text)
-
-            if not new_text:
-                log.add(_('Alias "%s" returned nothing'), text)
-                return
-
-            if new_text[:2] == "//":
-                new_text = new_text[1:]
-
-            self.frame.np.queue.append(slskmessages.MessageUser(self.user, auto_replace(new_text)))
-            widget.set_text("")
-            return
-
-        s = text.split(" ", 1)
-        cmd = s[0]
-        if len(s) == 2 and s[1]:
-            args = arg_self = s[1]
-        else:
-            args = ""
-            arg_self = self.user
-
-        if cmd in ("/alias", "/al"):
-
-            append_line(self.ChatScroll, add_alias(args), None, "")
-            if self.frame.np.config.sections["words"]["aliases"]:
-                self.frame.chatrooms.update_completions()
-                self.frame.privatechats.update_completions()
-
-        elif cmd in ("/unalias", "/un"):
-
-            append_line(self.ChatScroll, unalias(args), None, "")
-            if self.frame.np.config.sections["words"]["aliases"]:
-                self.frame.chatrooms.update_completions()
-                self.frame.privatechats.update_completions()
-
-        elif cmd in ("/join", "/j"):
-            if args:
-                self.frame.np.queue.append(slskmessages.JoinRoom(args))
-
-        elif cmd in ("/w", "/whois", "/info"):
-            self.frame.local_user_info_request(arg_self)
-            self.frame.change_main_page("userinfo")
-
-        elif cmd in ("/b", "/browse"):
-            self.frame.browse_user(arg_self)
-            self.frame.change_main_page("userbrowse")
-
-        elif cmd == "/ip":
-            self.frame.np.ip_requested.add(arg_self)
-            self.frame.np.queue.append(slskmessages.GetPeerAddress(arg_self))
-
-        elif cmd == "/pm":
-            if args:
-                self.frame.privatechats.send_message(args, show_user=True)
-                self.frame.change_main_page("private")
-
-        elif cmd in ("/m", "/msg"):
-            if args:
-                s = args.split(" ", 1)
-                user = s[0]
-                if len(s) == 2:
-                    msg = s[1]
-                else:
-                    msg = None
-
-                self.frame.privatechats.send_message(user, msg, show_user=True)
-                self.frame.change_main_page("private")
-
-        elif cmd in ("/s", "/search"):
-            if args:
-                self.frame.searches.do_search(args, 0)
-                self.frame.on_search(None)
-                self.frame.change_main_page("search")
-
-        elif cmd in ("/us", "/usearch"):
-            if args:
-                self.frame.searches.do_search(args, 3, [self.user])
-                self.frame.on_search(None)
-                self.frame.change_main_page("search")
-
-        elif cmd in ("/rs", "/rsearch"):
-            if args:
-                self.frame.searches.do_search(args, 1)
-                self.frame.on_search(None)
-                self.frame.change_main_page("search")
-
-        elif cmd in ("/bs", "/bsearch"):
-            if args:
-                self.frame.searches.do_search(args, 2)
-                self.frame.on_search(None)
-                self.frame.change_main_page("search")
-
-        elif cmd in ("/ad", "/add", "/buddy"):
-            if args:
-                self.frame.userlist.add_to_list(args)
-
-        elif cmd in ("/rem", "/unbuddy"):
-            if args:
-                self.frame.userlist.remove_from_list(args)
-
-        elif cmd == "/ban":
-            if args:
-                self.frame.np.network_filter.ban_user(args)
-
-        elif cmd == "/ignore":
-            if args:
-                self.frame.np.network_filter.ignore_user(args)
-
-        elif cmd == "/ignoreip":
-            if args:
-                self.frame.np.network_filter.ignore_ip(args)
-
-        elif cmd == "/unban":
-            if args:
-                self.frame.np.network_filter.unban_user(args)
-
-        elif cmd == "/unignore":
-            if args:
-                self.frame.np.network_filter.unignore_user(args)
-
-        elif cmd == "/ctcpversion":
-            if arg_self:
-                self.frame.privatechats.send_message(arg_self, CTCP_VERSION, show_user=True, bytestring=True)
-
-        elif cmd in ("/clear", "/cl"):
-            self.ChatScroll.get_buffer().set_text("")
-
-        elif cmd in ("/a", "/away"):
-            self.frame.on_away(None)
-
-        elif cmd in ("/q", "/quit", "/exit"):
-            self.frame.on_quit(None)
-            return
-
-        elif cmd in ("/c", "/close"):
-            self.on_close(None)
-
-        elif cmd == "/now":
-            self.display_now_playing()
-
-        elif cmd == "/rescan":
-            # Rescan public shares if needed
-            if not self.frame.np.config.sections["transfers"]["friendsonly"] and self.np.config.sections["transfers"]["shared"]:
-                self.frame.on_rescan()
-
-            # Rescan buddy shares if needed
-            if self.frame.np.config.sections["transfers"]["enablebuddyshares"]:
-                self.frame.on_buddy_rescan()
-
-        elif cmd[:1] == "/" and self.frame.np.pluginhandler.trigger_private_command_event(self.user, cmd[1:], args):
-            pass
-
-        elif cmd[:1] == "/" and cmd != "/me" and cmd[:2] != "//":
-            log.add(_("Command %s is not recognized"), text)
-            return
-
-        else:
-            if text[:2] == "//":
-                text = text[1:]
-
-            if self.chats.connected:
-                self.send_message(text)
-                widget.set_text("")
-
-            return
-
-        widget.set_text("")
-
-    def display_now_playing(self):
-        self.frame.np.now_playing.display_now_playing(callback=self.send_message)
-
     def update_visuals(self):
 
-        for widget in self.__dict__.values():
+        for widget in list(self.__dict__.values()):
             update_widget_visuals(widget, update_text_tags=False)
 
     def create_tag(self, buffer, color):
@@ -823,7 +585,7 @@ class PrivateChat:
         self.tag_username = self.create_tag(buffer, color)
 
         if self.chats.connected:
-            if self.frame.away and self.frame.np.config.sections["ui"]["showaway"]:
+            if self.frame.away and config.sections["ui"]["showaway"]:
                 self.tag_my_username = self.create_tag(buffer, "useraway")
             else:
                 self.tag_my_username = self.create_tag(buffer, "useronline")
@@ -841,7 +603,7 @@ class PrivateChat:
         update_tag_visuals(self.tag_username, color)
 
         if self.chats.connected:
-            if self.frame.away and self.frame.np.config.sections["ui"]["showaway"]:
+            if self.frame.away and config.sections["ui"]["showaway"]:
                 update_tag_visuals(self.tag_my_username, "useraway")
             else:
                 update_tag_visuals(self.tag_my_username, "useronline")
@@ -858,70 +620,19 @@ class PrivateChat:
         color = get_user_status_color(self.status)
         update_tag_visuals(self.tag_username, color)
 
-    def get_completion_list(self, ix=0, text="", clist=None):
-
-        config = self.frame.np.config.sections["words"]
-
-        completion = self.ChatLine.get_completion()
-        completion.set_popup_single_match(not config["onematch"])
-        completion.set_minimum_key_length(config["characters"])
-
-        liststore = completion.get_model()
-        liststore.clear()
-
-        if clist is None:
-            clist = []
-
-        if not config["tab"]:
-            return
-
-        # no duplicates
-        def _combilower(x):
-            try:
-                return str.lower(x)
-            except Exception:
-                return str.lower(x)
-
-        clist = list(set(clist))
-        clist.sort(key=_combilower)
-
-        completion.set_popup_completion(False)
-
-        if config["dropdown"]:
-            for word in clist:
-                liststore.append([word])
-
-            completion.set_popup_completion(True)
-
-        self.clist = clist
-
-    def on_key_press(self, widget, event):
-
-        keycode = event.hardware_keycode
-
-        if keycode not in keyval_to_hardware_keycode(Gdk.KEY_Tab):
-            return False
-
-        config = self.frame.np.config.sections["words"]
-        if not config["tab"]:
-            return False
-
-        ix = widget.get_position()
-        text = widget.get_text()[:ix].split(" ")[-1]
-        preix = ix - len(text)
-
-        completion, single = get_completion(text, self.chats.clist)
-
-        if completion:
-            widget.delete_text(preix, ix)
-            widget.insert_text(completion, ix)
-            widget.set_position(preix + len(completion))
-
-        widget.stop_emission_by_name("key_press_event")
-        return True
-
-    def on_close(self, widget):
+    def on_close(self, *args):
         self.chats.remove_tab(self)
 
-    def on_close_all_tabs(self, widget):
+    def on_close_all_tabs(self, *args):
         self.chats.remove_all_pages()
+
+    def set_completion_list(self, completion_list):
+
+        # Tab-complete the recepient username
+        completion_list.append(self.user)
+
+        # No duplicates
+        completion_list = list(set(completion_list))
+        completion_list.sort(key=lambda v: v.lower())
+
+        self.entry.set_completion_list(completion_list)
