@@ -18,10 +18,13 @@
 
 import os
 
+from gi.repository import Gdk
+
 from pynicotine import slskmessages
+from pynicotine.config import config
 from pynicotine.gtkgui.utils import append_line
 from pynicotine.gtkgui.utils import load_ui_elements
-from pynicotine.gtkgui.utils import update_widget_visuals
+from pynicotine.gtkgui.widgets.theme import update_widget_visuals
 
 
 class RoomWall:
@@ -34,19 +37,18 @@ class RoomWall:
         load_ui_elements(self, os.path.join(self.frame.gui_dir, "ui", "dialogs", "roomwall.ui"))
         self.RoomWallDialog.set_transient_for(frame.MainWindow)
 
-    def on_set_room_wall_message(self, widget, *args):
+    def on_set_room_wall_message(self, *args):
 
-        result = widget.get_text()
-        widget.set_text("")
+        result = self.RoomWallEntry.get_text()
+        self.RoomWallEntry.set_text("")
 
-        config = self.frame.np.config.sections
         room_name = self.room.room
 
         self.frame.np.queue.append(slskmessages.RoomTickerSet(room_name, result))
 
         self.RoomWallList.get_buffer().set_text("")
 
-        login = config["server"]["login"]
+        login = config.sections["server"]["login"]
 
         if result:
             append_line(self.RoomWallList, "[%s] %s" % (login, result), showstamp=False, scroll=False)
@@ -56,10 +58,10 @@ class RoomWall:
 
     def update_visuals(self):
 
-        for widget in self.__dict__.values():
+        for widget in list(self.__dict__.values()):
             update_widget_visuals(widget)
 
-    def hide(self, widget=None, event=None):
+    def hide(self, *args):
 
         self.RoomWallList.get_buffer().set_text("")
         self.RoomWallDialog.hide()
@@ -69,7 +71,7 @@ class RoomWall:
 
         tickers = self.room.tickers.get_tickers()
         append_line(self.RoomWallList, "%s" % ("\n".join(["[%s] %s" % (user, msg) for (user, msg) in tickers])), showstamp=False, scroll=False)
-        self.RoomWallDialog.show()
+        self.RoomWallDialog.present_with_time(Gdk.CURRENT_TIME)
 
 
 class Tickers:
