@@ -26,35 +26,39 @@ import os
 import re
 import sre_constants
 
+from collections import defaultdict
+
 from gi.repository import Gdk
 from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gtk
 
 from pynicotine import slskmessages
+from pynicotine.config import config
 from pynicotine.geoip.countrycodes import code2name
-from pynicotine.gtkgui.dialogs import choose_dir
 from pynicotine.gtkgui.fileproperties import FileProperties
-from pynicotine.gtkgui.utils import collapse_treeview
-from pynicotine.gtkgui.utils import humanize
-from pynicotine.gtkgui.utils import human_size
-from pynicotine.gtkgui.utils import human_speed
-from pynicotine.gtkgui.utils import IconNotebook
-from pynicotine.gtkgui.utils import initialise_columns
+from pynicotine.gtkgui.utils import copy_file_url
 from pynicotine.gtkgui.utils import keyval_to_hardware_keycode
 from pynicotine.gtkgui.utils import load_ui_elements
-from pynicotine.gtkgui.utils import PopupMenu
-from pynicotine.gtkgui.utils import save_columns
-from pynicotine.gtkgui.utils import select_user_row_iter
-from pynicotine.gtkgui.utils import set_widget_fg_bg_css
-from pynicotine.gtkgui.utils import set_treeview_selected_row
-from pynicotine.gtkgui.utils import show_country_tooltip
-from pynicotine.gtkgui.utils import show_file_path_tooltip
 from pynicotine.gtkgui.utils import triggers_context_menu
-from pynicotine.gtkgui.utils import update_widget_visuals
+from pynicotine.gtkgui.widgets.filechooser import choose_dir
+from pynicotine.gtkgui.widgets.iconnotebook import IconNotebook
+from pynicotine.gtkgui.widgets.popupmenu import PopupMenu
+from pynicotine.gtkgui.widgets.theme import set_widget_fg_bg_css
+from pynicotine.gtkgui.widgets.treeview import collapse_treeview
+from pynicotine.gtkgui.widgets.treeview import initialise_columns
+from pynicotine.gtkgui.widgets.treeview import save_columns
+from pynicotine.gtkgui.widgets.treeview import select_user_row_iter
+from pynicotine.gtkgui.widgets.treeview import set_treeview_selected_row
+from pynicotine.gtkgui.widgets.treeview import show_country_tooltip
+from pynicotine.gtkgui.widgets.treeview import show_file_path_tooltip
+from pynicotine.gtkgui.widgets.theme import update_widget_visuals
 from pynicotine.gtkgui.wishlist import WishList
 from pynicotine.logfacility import log
 from pynicotine.utils import get_result_bitrate_length
+from pynicotine.utils import humanize
+from pynicotine.utils import human_size
+from pynicotine.utils import human_speed
 
 
 class Searches(IconNotebook):
@@ -66,18 +70,14 @@ class Searches(IconNotebook):
         self.searches = {}
         self.usersearches = {}
         self.users = {}
-        self.maxdisplayedresults = self.frame.np.config.sections['searches']["max_displayed_results"]
-        self.maxstoredresults = self.frame.np.config.sections['searches']["max_stored_results"]
-
-        ui = self.frame.np.config.sections["ui"]
 
         IconNotebook.__init__(
             self,
             self.frame.images,
-            angle=ui["labelsearch"],
-            tabclosers=ui["tabclosers"],
-            show_hilite_image=self.frame.np.config.sections["notifications"]["notification_tab_icons"],
-            reorderable=ui["tab_reorderable"],
+            angle=config.sections["ui"]["labelsearch"],
+            tabclosers=config.sections["ui"]["tabclosers"],
+            show_hilite_image=config.sections["notifications"]["notification_tab_icons"],
+            reorderable=config.sections["ui"]["tab_reorderable"],
             notebookraw=self.frame.SearchNotebookRaw
         )
 
@@ -93,7 +93,7 @@ class Searches(IconNotebook):
         Add search history to SearchCombo later and connect Wishlist,
         after widgets have been created.
         """
-        items = self.frame.np.config.sections["searches"]["history"]
+        items = config.sections["searches"]["history"]
         templist = []
 
         for i in items:
@@ -173,7 +173,7 @@ class Searches(IconNotebook):
         # Repopulate the combo list
         self.frame.SearchCombo.remove_all()
 
-        items = self.frame.np.config.sections["searches"]["history"]
+        items = config.sections["searches"]["history"]
 
         for i in items:
             if not isinstance(i, str):
@@ -191,21 +191,21 @@ class Searches(IconNotebook):
 
         self.frame.SearchEntry.set_text("")
 
-        self.frame.np.config.sections["searches"]["history"] = []
-        self.frame.np.config.write_configuration()
+        config.sections["searches"]["history"] = []
+        config.write_configuration()
 
         self.frame.SearchCombo.remove_all()
 
     def clear_filter_history(self):
 
         # Clear filter history in config
-        self.frame.np.config.sections["searches"]["filterin"] = []
-        self.frame.np.config.sections["searches"]["filterout"] = []
-        self.frame.np.config.sections["searches"]["filtertype"] = []
-        self.frame.np.config.sections["searches"]["filtersize"] = []
-        self.frame.np.config.sections["searches"]["filterbr"] = []
-        self.frame.np.config.sections["searches"]["filtercc"] = []
-        self.frame.np.config.write_configuration()
+        config.sections["searches"]["filterin"] = []
+        config.sections["searches"]["filterout"] = []
+        config.sections["searches"]["filtertype"] = []
+        config.sections["searches"]["filtersize"] = []
+        config.sections["searches"]["filterbr"] = []
+        config.sections["searches"]["filtercc"] = []
+        config.write_configuration()
 
         # Update filters in search tabs
         for id in self.searches.values():
@@ -280,7 +280,7 @@ class Searches(IconNotebook):
         counter = len(search["tab"].all_data) + 1
 
         # No more things to add because we've reached the max_stored_results limit
-        if counter > self.maxstoredresults:
+        if counter > config.sections["searches"]["max_stored_results"]:
             return
 
         search["tab"].add_user_results(msg, username, country)
@@ -290,7 +290,7 @@ class Searches(IconNotebook):
         if tab.id in self.searches:
             search = self.searches[tab.id]
 
-            if tab.text not in self.frame.np.config.sections["server"]["autosearch"]:
+            if tab.text not in config.sections["server"]["autosearch"]:
                 del self.searches[tab.id]
             else:
                 search["tab"] = None
@@ -308,19 +308,17 @@ class Searches(IconNotebook):
         self.wish_list.update_visuals()
 
     def save_columns(self):
+        """ Save the treeview state of the currently selected tab """
 
-        page_num = self.get_current_page()
+        current_page = self.get_nth_page(self.get_current_page())
 
-        if page_num is not None:
-            page = self.get_nth_page(page_num)
+        for search in self.searches.values():
+            if search["tab"] is None:
+                continue
 
-            for search in self.searches.values():
-                if search["tab"] is None:
-                    continue
-
-                if search["tab"].Main == page:
-                    search["tab"].save_columns()
-                    break
+            if search["tab"].Main == current_page:
+                search["tab"].save_columns()
+                break
 
     def get_search_id(self, child):
 
@@ -346,15 +344,7 @@ class Searches(IconNotebook):
             log.add_warning(_("Search ID was none when clicking tab"))
             return False
 
-        menu = PopupMenu(self.frame)
-        menu.setup(
-            ("#" + _("Copy Search Term"), self.searches[search_id]["tab"].on_copy_search_term),
-            ("", None),
-            ("#" + _("Clear All Results"), self.searches[search_id]["tab"].on_clear),
-            ("#" + _("Close All Tabs"), self.searches[search_id]["tab"].on_close_all_tabs),
-            ("#" + _("_Close Tab"), self.searches[search_id]["tab"].on_close)
-        )
-
+        menu = self.searches[search_id]["tab"].tab_menu
         menu.popup()
         return True
 
@@ -463,14 +453,15 @@ class Search:
 
         """ Filters """
 
-        self.ShowFilters.set_active(self.frame.np.config.sections["searches"]["filters_visible"])
+        self.ShowFilters.set_active(config.sections["searches"]["filters_visible"])
         self.populate_filters()
 
         """ Popup """
 
-        self.popup_menu_users = PopupMenu(self.frame, False)
-        self.popup_menu = popup = PopupMenu(self.frame)
-        popup.setup(
+        self.popup_menu_users = PopupMenu(self.frame)
+
+        self.popup_menu = PopupMenu(self.frame)
+        self.popup_menu.setup(
             ("#" + "selected_files", None),
             ("", None),
             ("#" + _("_Download File(s)"), self.on_download_files),
@@ -484,13 +475,22 @@ class Search:
             ("#" + _("Copy _URL"), self.on_copy_url),
             ("#" + _("Copy Folder U_RL"), self.on_copy_dir_url),
             ("", None),
-            (1, _("User(s)"), self.popup_menu_users, self.on_popup_menu_users)
+            (">" + _("User(s)"), self.popup_menu_users)
+        )
+
+        self.tab_menu = PopupMenu(self.frame)
+        self.tab_menu.setup(
+            ("#" + _("Copy Search Term"), self.on_copy_search_term),
+            ("", None),
+            ("#" + _("Clear All Results"), self.on_clear),
+            ("#" + _("Close All Tabs"), self.on_close_all_tabs),
+            ("#" + _("_Close Tab"), self.on_close)
         )
 
         """ Grouping """
 
-        self.ResultGrouping.set_active(self.frame.np.config.sections["searches"]["group_searches"])
-        self.ExpandButton.set_active(self.frame.np.config.sections["searches"]["expand_searches"])
+        self.ResultGrouping.set_active(config.sections["searches"]["group_searches"])
+        self.ExpandButton.set_active(config.sections["searches"]["expand_searches"])
 
     def on_tooltip(self, widget, x, y, keyboard_mode, tooltip):
 
@@ -509,9 +509,9 @@ class Search:
                          self.FilterBitrate, self.FilterCountry):
             combobox.remove_all()
 
-        if set_default_filters and self.frame.np.config.sections["searches"]["enablefilters"]:
+        if set_default_filters and config.sections["searches"]["enablefilters"]:
 
-            sfilter = self.frame.np.config.sections["searches"]["defilter"]
+            sfilter = config.sections["searches"]["defilter"]
 
             self.FilterInEntry.set_text(str(sfilter[0]))
             self.FilterOutEntry.set_text(str(sfilter[1]))
@@ -536,24 +536,22 @@ class Search:
         for i in ['flac|wav|ape|aiff|wv|cue', 'mp3|m4a|aac|ogg|opus|wma', '!mp3']:
             self.FilterType.append_text(i)
 
-        s_config = self.frame.np.config.sections["searches"]
-
-        for i in s_config["filterin"]:
+        for i in config.sections["searches"]["filterin"]:
             self.add_combo(self.FilterIn, i, True)
 
-        for i in s_config["filterout"]:
+        for i in config.sections["searches"]["filterout"]:
             self.add_combo(self.FilterOut, i, True)
 
-        for i in s_config["filtersize"]:
+        for i in config.sections["searches"]["filtersize"]:
             self.add_combo(self.FilterSize, i, True)
 
-        for i in s_config["filterbr"]:
+        for i in config.sections["searches"]["filterbr"]:
             self.add_combo(self.FilterBitrate, i, True)
 
-        for i in s_config["filtercc"]:
+        for i in config.sections["searches"]["filtercc"]:
             self.add_combo(self.FilterCountry, i, True)
 
-        for i in s_config["filtertype"]:
+        for i in config.sections["searches"]["filtertype"]:
             self.add_combo(self.FilterType, i, True)
 
     def focus_combobox(self, button):
@@ -617,12 +615,12 @@ class Search:
             imdl = "N"
 
         color_id = (imdl == "Y" and "search" or "searchq")
-        color = self.frame.np.config.sections["ui"][color_id] or None
+        color = config.sections["ui"][color_id] or None
 
         h_queue = humanize(inqueue)
 
         update_ui = False
-        maxstoredresults = self.searches.maxstoredresults
+        maxstoredresults = config.sections["searches"]["max_stored_results"]
 
         for result in msg.list:
 
@@ -696,7 +694,7 @@ class Search:
 
         self.all_data.append(row)
 
-        if self.numvisibleresults >= self.searches.maxdisplayedresults:
+        if self.numvisibleresults >= config.sections["searches"]["max_displayed_results"]:
             return False
 
         if not self.check_filter(row):
@@ -992,7 +990,7 @@ class Search:
         self.numvisibleresults = 0
 
         for row in self.all_data:
-            if self.numvisibleresults >= self.searches.maxdisplayedresults:
+            if self.numvisibleresults >= config.sections["searches"]["max_displayed_results"]:
                 break
 
             if self.check_filter(row):
@@ -1002,42 +1000,32 @@ class Search:
         self.update_result_counter()
         self.update_filter_counter(self.active_filter_count)
 
-    def on_popup_menu_users(self, widget):
-
-        self.select_results()
+    def populate_popup_menu_users(self):
 
         self.popup_menu_users.clear()
 
-        if self.selected_users:
-
-            items = []
-
-            for user in self.selected_users:
-                popup = PopupMenu(self.frame, False)
-                popup.setup_user_menu(user)
-                popup.append_item(("", None))
-                popup.append_item(("#" + _("Select User's Transfers"), self.on_select_user_results))
-
-                items.append((1, user, popup, self.on_popup_menu_user, popup))
-
-            self.popup_menu_users.setup(*items)
-
-        return True
-
-    def on_popup_menu_user(self, widget, popup=None):
-
-        if popup is None:
-            return
-
-        popup.toggle_user_items()
-        return True
-
-    def on_select_user_results(self, widget):
-
         if not self.selected_users:
             return
 
-        selected_user = widget.get_parent().user
+        for user in self.selected_users:
+            popup = PopupMenu(self.frame)
+            popup.setup_user_menu(user)
+            popup.setup(
+                ("", None),
+                ("#" + _("Select User's Transfers"), self.on_select_user_results, user)
+            )
+
+            popup.toggle_user_items()
+            self.popup_menu_users.setup(
+                (">" + user, popup)
+            )
+
+    def on_select_user_results(self, *args):
+
+        if not self.selected_users:
+            return
+
+        selected_user = args[-1]
 
         sel = self.ResultsList.get_selection()
         fmodel = self.ResultsList.get_model()
@@ -1051,54 +1039,55 @@ class Search:
 
     def select_results(self):
 
-        self.selected_results = set()
-        self.selected_users = set()
+        self.selected_results = []
+        self.selected_users = []
         self.selected_files_count = 0
 
-        self.ResultsList.get_selection().selected_foreach(self.selected_results_callback)
+        model, paths = self.ResultsList.get_selection().get_selected_rows()
+
+        for path in paths:
+            iterator = model.get_iter(path)
+            user = model.get_value(iterator, 1)
+
+            if user is None:
+                continue
+
+            if user not in self.selected_users:
+                self.selected_users.append(user)
+
+            filepath = model.get_value(iterator, 12)
+
+            if not filepath:
+                # Result is not a file or directory, don't add it
+                continue
+
+            bitrate = model.get_value(iterator, 9)
+            length = model.get_value(iterator, 10)
+            size = model.get_value(iterator, 14)
+
+            self.selected_results.append((user, filepath, size, bitrate, length))
+
+            filename = model.get_value(iterator, 7)
+
+            if filename:
+                self.selected_files_count += 1
 
     def update_result_counter(self):
         self.Counter.set_text(str(self.numvisibleresults))
 
     def update_visuals(self):
 
-        for widget in self.__dict__.values():
+        for widget in list(self.__dict__.values()):
             update_widget_visuals(widget, list_font_target="searchfont")
 
     def save_columns(self):
         save_columns("file_search", self.ResultsList.get_columns())
 
-    def selected_results_callback(self, model, path, iterator):
-
-        user = model.get_value(iterator, 1)
-
-        if user is None:
-            return
-
-        self.selected_users.add(user)
-
-        filepath = model.get_value(iterator, 12)
-
-        if filepath == "":
-            # Result is not a file or directory, don't add it
-            return
-
-        bitrate = model.get_value(iterator, 9)
-        length = model.get_value(iterator, 10)
-        size = model.get_value(iterator, 14)
-
-        self.selected_results.add((user, filepath, size, bitrate, length))
-
-        filename = model.get_value(iterator, 7)
-
-        if filename:
-            self.selected_files_count += 1
-
     def on_list_clicked(self, widget, event):
 
         if triggers_context_menu(event):
             set_treeview_selected_row(widget, event)
-            return self.on_popup_menu(widget)
+            return self.on_popup_menu()
 
         pathinfo = widget.get_path_at_pos(event.x, event.y)
 
@@ -1107,7 +1096,7 @@ class Search:
 
         elif event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
             self.select_results()
-            self.on_download_files(widget)
+            self.on_download_files()
             self.ResultsList.get_selection().unselect_all()
             return True
 
@@ -1119,7 +1108,7 @@ class Search:
 
         if event.get_state() & Gdk.ModifierType.CONTROL_MASK and \
                 event.hardware_keycode in keyval_to_hardware_keycode(Gdk.KEY_c):
-            self.on_copy_file_path(widget)
+            self.on_copy_file_path()
         else:
             # No key match, continue event
             return False
@@ -1127,23 +1116,24 @@ class Search:
         widget.stop_emission_by_name("key_press_event")
         return True
 
-    def on_popup_menu(self, widget):
+    def on_popup_menu(self, *args):
 
         self.select_results()
 
-        items = self.popup_menu.get_items()
+        actions = self.popup_menu.get_actions()
         users = len(self.selected_users) > 0
         files = len(self.selected_results) > 0
 
         for i in (_("_Download File(s)"), _("Download File(s) _To..."), _("File _Properties"),
                   _("Copy _URL")):
-            items[i].set_sensitive(False)
+            actions[i].set_enabled(False)
 
         for i in (_("Download _Folder(s)"), _("Download F_older(s) To..."), _("_Browse Folder"),
                   _("Copy _File Path"), _("Copy Folder U_RL")):
-            items[i].set_sensitive(files)
+            actions[i].set_enabled(files)
 
-        items[_("User(s)")].set_sensitive(users)
+        actions[_("User(s)")].set_enabled(users)
+        self.populate_popup_menu_users()
 
         for result in self.selected_results:
             if not result[1].endswith('\\'):
@@ -1151,17 +1141,16 @@ class Search:
 
                 for i in (_("_Download File(s)"), _("Download File(s) _To..."), _("File _Properties"),
                           _("Copy _URL")):
-                    items[i].set_sensitive(True)
+                    actions[i].set_enabled(True)
 
                 break
 
-        items["selected_files"].set_sensitive(False)
-        items["selected_files"].set_label(_("%s File(s) Selected") % self.selected_files_count)
+        self.popup_menu.set_num_selected_files(self.selected_files_count)
 
         self.popup_menu.popup()
         return True
 
-    def on_browse_folder(self, widget):
+    def on_browse_folder(self, *args):
 
         requested_folders = set()
 
@@ -1173,12 +1162,22 @@ class Search:
                 self.frame.browse_user(user, folder)
                 requested_folders.add(folder)
 
-    def selected_results_all_data(self, model, path, iterator, data):
+    def on_file_properties(self, *args):
 
-        filename = model.get_value(iterator, 7)
+        if not self.frame.np.transfers:
+            return
+
+        data = []
+        model, paths = self.ResultsList.get_selection().get_selected_rows()
+
+        for path in paths:
+            iterator = model.get_iter(path)
+            filename = model.get_value(iterator, 7)
+
+            # We only want to see the metadata of files, not directories
+            if not filename:
+                continue
 
-        # We only want to see the metadata of files, not directories
-        if filename != "":
             num = model.get_value(iterator, 0)
             user = model.get_value(iterator, 1)
             immediate = model.get_value(iterator, 3)
@@ -1207,18 +1206,10 @@ class Search:
                 "country": country
             })
 
-    def on_file_properties(self, widget):
-
-        if not self.frame.np.transfers:
-            return
-
-        data = []
-        self.ResultsList.get_selection().selected_foreach(self.selected_results_all_data, data)
-
-        if data:
+        if paths:
             FileProperties(self.frame, data).show()
 
-    def on_download_files(self, widget, prefix=""):
+    def on_download_files(self, *args, prefix=""):
 
         if not self.frame.np.transfers:
             return
@@ -1228,81 +1219,97 @@ class Search:
             if not file[1].endswith('\\'):
                 self.frame.np.transfers.get_file(file[0], file[1], prefix, size=file[2], bitrate=file[3], length=file[4], checkduplicate=True)
 
-    def on_download_files_to(self, widget):
+    def on_download_files_to_selected(self, selected, data):
+        self.on_download_files(prefix=selected)
 
-        folder = choose_dir(self.frame.MainWindow, self.frame.np.config.sections["transfers"]["downloaddir"], multichoice=False)
+    def on_download_files_to(self, *args):
 
-        if folder is None:
+        choose_dir(
+            parent=self.frame.MainWindow,
+            callback=self.on_download_files_to_selected,
+            initialdir=config.sections["transfers"]["downloaddir"],
+            multichoice=False
+        )
+
+    def on_download_folders(self, *args, download_location=""):
+
+        if not self.frame.np.transfers:
             return
 
-        for folders in folder:
-            self.on_download_files(widget, folders)
-            break
+        if download_location:
+            """ Custom download location specified, remember it when peer sends a folder
+            contents reply """
 
-    def on_download_folders(self, widget):
-
-        requested_folders = {}
+            requested_folders = self.frame.np.transfers.requested_folders
+        else:
+            requested_folders = defaultdict(dict)
 
         for i in self.selected_results:
-
             user = i[0]
             folder = i[1].rsplit('\\', 1)[0]
 
-            if user not in requested_folders:
-                requested_folders[user] = []
-
-            if folder not in requested_folders[user]:
+            if folder in requested_folders[user]:
                 """ Ensure we don't send folder content requests for a folder more than once,
                 e.g. when several selected resuls belong to the same folder. """
+                continue
 
-                self.frame.np.send_message_to_peer(user, slskmessages.FolderContentsRequest(None, folder))
-                requested_folders[user].append(folder)
+            requested_folders[user][folder] = download_location
 
-    def on_download_folders_to(self, widget):
+            # First queue the visible search results
+            files = []
+            for row in self.all_data:
 
-        directories = choose_dir(self.frame.MainWindow, self.frame.np.config.sections["transfers"]["downloaddir"], multichoice=False)
+                # Find the wanted directory
+                if folder != row[12].rsplit('\\', 1)[0]:
+                    continue
 
-        if not directories:
-            return
+                destination = self.frame.np.transfers.get_folder_destination(user, folder)
+                counter, user, flag, immediatedl, h_speed, h_queue, directory, filename, h_size, h_bitrate, h_length, bitrate, fullpath, country, size, speed, queue, length, color = row
+                files.append((user, fullpath, destination, size.get_uint64(), bitrate.get_uint64(), length.get_uint64()))
 
-        destination = directories[0]
+            if config.sections["transfers"]["reverseorder"]:
+                files.sort(key=lambda x: x[1], reverse=True)
 
-        for i in self.selected_results:
+            for file in files:
+                user, fullpath, destination, size, bitrate, length = file
 
-            user = i[0]
-            folder = i[1].rsplit('\\', 1)[0]
+                self.frame.np.transfers.get_file(
+                    user, fullpath, destination,
+                    size=size, bitrate=bitrate, length=length, checkduplicate=True
+                )
 
-            if user not in self.frame.np.requested_folders:
-                self.frame.np.requested_folders[user] = {}
+            # Ask for the rest of the files in the folder
+            self.frame.np.send_message_to_peer(user, slskmessages.FolderContentsRequest(None, folder))
 
-            if folder not in self.frame.np.requested_folders[user]:
-                """ Ensure we don't send folder content requests for a folder more than once,
-                e.g. when several selected resuls belong to the same folder. """
+    def on_download_folders_to_selected(self, selected, data):
+        self.on_download_folders(download_location=selected)
 
-                self.frame.np.requested_folders[user][folder] = destination
-                self.frame.np.send_message_to_peer(user, slskmessages.FolderContentsRequest(None, folder))
+    def on_download_folders_to(self, *args):
 
-    def on_copy_file_path(self, widget):
+        choose_dir(
+            parent=self.frame.MainWindow,
+            callback=self.on_download_folders_to_selected,
+            initialdir=config.sections["transfers"]["downloaddir"],
+            multichoice=False
+        )
 
-        if not self.selected_results:
-            return
+    def on_copy_file_path(self, *args):
 
-        user, path = next(iter(self.selected_results))[:2]
-        self.frame.clip.set_text(path, -1)
+        if self.selected_results:
+            user, path = self.selected_results[0][:2]
+            self.frame.clipboard.set_text(path, -1)
 
-    def on_copy_url(self, widget):
-        user, path = next(iter(self.selected_results))[:2]
-        self.frame.set_clipboard_url(user, path)
+    def on_copy_url(self, *args):
 
-    def on_copy_dir_url(self, widget):
+        if self.selected_results:
+            user, path = self.selected_results[0][:2]
+            copy_file_url(user, path, self.frame.clipboard)
 
-        user, path = next(iter(self.selected_results))[:2]
-        path = "\\".join(path.split("\\")[:-1])
+    def on_copy_dir_url(self, *args):
 
-        if path[:-1] != "/":
-            path += "/"
-
-        self.frame.set_clipboard_url(user, path)
+        if self.selected_results:
+            user, path = self.selected_results[0][:2]
+            copy_file_url(user, path.rsplit('\\', 1)[0] + '\\', self.frame.clipboard)
 
     def on_group(self, widget):
 
@@ -1311,7 +1318,7 @@ class Search:
         active = widget.get_active()
 
         self.ResultsList.set_show_expanders(active)
-        self.frame.np.config.sections["searches"]["group_searches"] = active
+        config.sections["searches"]["group_searches"] = active
         self.cols["id"].set_visible(not active)
         self.ExpandButton.set_visible(active)
 
@@ -1326,16 +1333,16 @@ class Search:
             collapse_treeview(self.ResultsList, self.ResultGrouping.get_active_id())
             self.expand.set_from_icon_name("go-down-symbolic", Gtk.IconSize.BUTTON)
 
-        self.frame.np.config.sections["searches"]["expand_searches"] = active
+        config.sections["searches"]["expand_searches"] = active
 
     def on_toggle_filters(self, widget):
 
         visible = widget.get_active()
         self.FiltersContainer.set_visible(visible)
-        self.frame.np.config.sections["searches"]["filters_visible"] = visible
+        config.sections["searches"]["filters_visible"] = visible
 
-    def on_copy_search_term(self, widget):
-        self.frame.clip.set_text(self.text, -1)
+    def on_copy_search_term(self, *args):
+        self.frame.clipboard.set_text(self.text, -1)
 
     def on_toggle_remember(self, widget):
 
@@ -1354,7 +1361,7 @@ class Search:
             return None
 
         text = text.strip()
-        history = self.frame.np.config.sections["searches"][title]
+        history = config.sections["searches"][title]
 
         if text in history:
             history.remove(text)
@@ -1362,14 +1369,14 @@ class Search:
             del history[-1]
 
         history.insert(0, text)
-        self.frame.np.config.write_configuration()
+        config.write_configuration()
 
         self.add_combo(widget, text)
         widget.get_child().set_text(text)
 
         return text
 
-    def on_refilter(self, widget, *args):
+    def on_refilter(self, *args):
 
         if self.clearing_filters:
             return
@@ -1394,7 +1401,7 @@ class Search:
             else:
                 collapse_treeview(self.ResultsList, self.ResultGrouping.get_active_id())
 
-    def on_clear_filters(self, widget):
+    def on_clear_filters(self, *args):
 
         self.clearing_filters = True
 
@@ -1408,9 +1415,9 @@ class Search:
 
         self.clearing_filters = False
         self.FilterInEntry.grab_focus()
-        self.on_refilter(widget)
+        self.on_refilter()
 
-    def on_about_filters(self, widget):
+    def on_about_filters(self, *args):
 
         if not hasattr(self, "AboutSearchFiltersPopover"):
             load_ui_elements(self, os.path.join(self.frame.gui_dir, "ui", "popovers", "searchfilters.ui"))
@@ -1433,7 +1440,8 @@ class Search:
 
         self.FilterLabel.set_tooltip_text("%d active filter(s)" % count)
 
-    def on_clear(self, widget):
+    def on_clear(self, *args):
+
         self.all_data = []
         self.usersiters.clear()
         self.directoryiters.clear()
@@ -1443,8 +1451,8 @@ class Search:
         # Update number of visible results
         self.update_result_counter()
 
-    def on_close(self, widget):
+    def on_close(self, *args):
         self.searches.remove_tab(self)
 
-    def on_close_all_tabs(self, widget):
+    def on_close_all_tabs(self, *args):
         self.searches.remove_all_pages()
