@@ -31,19 +31,23 @@ from gi.repository import Gdk
 from gi.repository import GObject
 from gi.repository import Gtk
 
-from pynicotine.gtkgui.utils import collapse_treeview
-from pynicotine.gtkgui.utils import human_size
-from pynicotine.gtkgui.utils import human_speed
-from pynicotine.gtkgui.utils import initialise_columns
+from pynicotine.config import config
+from pynicotine.gtkgui.fileproperties import FileProperties
+from pynicotine.gtkgui.utils import copy_file_url
 from pynicotine.gtkgui.utils import keyval_to_hardware_keycode
 from pynicotine.gtkgui.utils import load_ui_elements
-from pynicotine.gtkgui.utils import PopupMenu
-from pynicotine.gtkgui.utils import save_columns
-from pynicotine.gtkgui.utils import select_user_row_iter
-from pynicotine.gtkgui.utils import set_treeview_selected_row
-from pynicotine.gtkgui.utils import show_file_path_tooltip
 from pynicotine.gtkgui.utils import triggers_context_menu
-from pynicotine.gtkgui.utils import update_widget_visuals
+from pynicotine.gtkgui.widgets.popupmenu import PopupMenu
+from pynicotine.gtkgui.widgets.theme import update_widget_visuals
+from pynicotine.gtkgui.widgets.treeview import collapse_treeview
+from pynicotine.gtkgui.widgets.treeview import initialise_columns
+from pynicotine.gtkgui.widgets.treeview import save_columns
+from pynicotine.gtkgui.widgets.treeview import select_user_row_iter
+from pynicotine.gtkgui.widgets.treeview import set_treeview_selected_row
+from pynicotine.gtkgui.widgets.treeview import show_file_path_tooltip
+from pynicotine.transfers import Transfer
+from pynicotine.utils import human_size
+from pynicotine.utils import human_speed
 
 
 class TransferList:
@@ -53,9 +57,9 @@ class TransferList:
         self.frame = frame
         self.type = type
 
-        load_ui_elements(self, os.path.join(self.frame.gui_dir, "ui", type + "s.ui"))
-        self.frame.__dict__[type + "svbox"].add(self.Main)
-        self.widget = widget = self.__dict__[type.title() + "List"]
+        load_ui_elements(self, os.path.join(frame.gui_dir, "ui", type + "s.ui"))
+        getattr(frame, type + "svbox").add(self.Main)
+        self.widget = widget = getattr(self, type.title() + "List")
 
         self.last_ui_update = self.last_save = 0
         self.list = []
@@ -73,7 +77,6 @@ class TransferList:
         self.statuses["Connection closed by peer"] = _("Connection closed by peer")
         self.statuses["Aborted"] = _("Aborted")
         self.statuses["Finished"] = _("Finished")
-        self.statuses["Paused"] = _("Paused")
         self.statuses["Filtered"] = _("Filtered")
         self.statuses["File not shared"] = _("File not shared")
         self.statuses["File not shared."] = _("File not shared")  # The official client sends a variant containing a dot
@@ -104,6 +107,7 @@ class TransferList:
             GObject.TYPE_UINT64,   # (15) time elapsed
             GObject.TYPE_UINT64,   # (16) file count
             GObject.TYPE_UINT64,   # (17) queue position
+            GObject.TYPE_PYOBJECT  # (18) transfer object
         )
 
         self.column_numbers = list(range(self.transfersmodel.get_n_columns()))
@@ -135,71 +139,84 @@ class TransferList:
 
         widget.set_model(self.transfersmodel)
 
-        self.group_dropdown = frame.__dict__["ToggleTree%ss" % self.type.title()]
-        self.expand_button = frame.__dict__["Expand%ss" % self.type.title()]
+        self.group_dropdown = getattr(frame, "ToggleTree%ss" % self.type.title())
+        self.expand_button = getattr(frame, "Expand%ss" % self.type.title())
 
         self.group_dropdown.connect("changed", self.on_toggle_tree)
-        self.group_dropdown.set_active(frame.np.config.sections["transfers"]["group%ss" % self.type])
+        self.group_dropdown.set_active(config.sections["transfers"]["group%ss" % self.type])
 
         self.expand_button.connect("toggled", self.on_expand_tree)
-        self.expand_button.set_active(frame.np.config.sections["transfers"]["%ssexpanded" % self.type])
+        self.expand_button.set_active(config.sections["transfers"]["%ssexpanded" % self.type])
+
+        self.popup_menu_users = PopupMenu(frame)
+        self.popup_menu_clear = PopupMenu(frame)
+
+        self.popup_menu = PopupMenu(frame)
+        self.popup_menu.setup(
+            ("#" + "selected_files", None),
+            ("", None),
+            ("#" + _("Send to _Player"), self.on_play_files),
+            ("#" + _("_Open Folder"), self.on_open_directory),
+            ("#" + _("File P_roperties"), self.on_file_properties),
+            ("", None),
+            ("#" + _("Copy _File Path"), self.on_copy_file_path),
+            ("#" + _("Copy _URL"), self.on_copy_url),
+            ("#" + _("Copy Folder URL"), self.on_copy_dir_url),
+            ("", None),
+            ("#" + _("_Search"), self.on_file_search),
+            (">" + _("User(s)"), self.popup_menu_users),
+            ("", None),
+            ("#" + _("_Retry"), self.on_retry_transfer),
+            ("#" + _("Abor_t"), self.on_abort_transfer),
+            ("#" + _("_Clear"), self.on_clear_transfer),
+            ("", None),
+            (">" + _("Clear Groups"), self.popup_menu_clear)
+        )
 
         self.update_visuals()
 
-    def save_columns(self):
-        save_columns(self.type, self.widget.get_columns())
-
-    def update_visuals(self):
-
-        for widget in self.__dict__.values():
-            update_widget_visuals(widget, list_font_target="transfersfont")
-
     def init_interface(self, list):
+
         self.list = list
         self.widget.set_sensitive(True)
         self.update()
 
-    def conn_close(self):
-        self.widget.set_sensitive(False)
-        self.list = []
-        self.clear()
-
-    def select_transfers(self):
-        self.selected_transfers = set()
-        self.selected_users = set()
-
-        self.widget.get_selection().selected_foreach(self.selected_transfers_callback)
-
-    def new_transfer_notification(self):
-        self.frame.request_tab_icon(self.tab_label)
-
-    def on_ban(self, widget):
-        self.select_transfers()
-
-        for user in self.selected_users:
-            self.frame.np.network_filter.ban_user(user)
-
-    def on_file_search(self, widget):
-
-        for transfer in self.selected_transfers:
-            self.frame.SearchEntry.set_text(transfer.filename.rsplit("\\", 1)[1])
-            self.frame.change_main_page("search")
-            break
-
     def rebuild_transfers(self):
+
         if self.frame.np.transfers is None:
             return
 
         self.clear()
         self.update()
 
-    def selected_transfers_callback(self, model, path, iterator):
+    def save_columns(self):
+        save_columns(self.type, self.widget.get_columns())
 
-        self.select_transfer(model, iterator, selectuser=True)
+    def update_visuals(self):
 
-        # If we're in grouping mode, select any transfers under the selected
-        # user or folder
-        self.select_child_transfers(model, model.iter_children(iterator))
+        for widget in list(self.__dict__.values()):
+            update_widget_visuals(widget, list_font_target="transfersfont")
+
+    def conn_close(self):
+
+        self.widget.set_sensitive(False)
+        self.list = []
+        self.clear()
+
+    def select_transfers(self):
+
+        self.selected_transfers = set()
+        self.selected_users = set()
+
+        model, paths = self.widget.get_selection().get_selected_rows()
+
+        for path in paths:
+            iterator = model.get_iter(path)
+            self.select_transfer(model, iterator, select_user=True)
+
+            # If we're in grouping mode, select any transfers under the selected
+            # user or folder
+            self.select_child_transfers(model, model.iter_children(iterator))
 
     def select_child_transfers(self, model, iterator):
 
@@ -208,19 +225,37 @@ class TransferList:
             self.select_child_transfers(model, model.iter_children(iterator))
             iterator = model.iter_next(iterator)
 
-    def select_transfer(self, model, iterator, selectuser=False):
+    def select_transfer(self, model, iterator, select_user=False):
 
         user = model.get_value(iterator, 0)
-        filepath = model.get_value(iterator, 10)
+        transfer = model.get_value(iterator, 18)
 
-        for i in self.list:
-            if i.user == user and i.filename == filepath:
-                self.selected_transfers.add(i)
-                break
+        if isinstance(transfer, Transfer):
+            self.selected_transfers.add(transfer)
 
-        if selectuser:
+        if select_user:
             self.selected_users.add(user)
 
+    def new_transfer_notification(self):
+        self.frame.request_tab_icon(self.tab_label)
+
+    def on_ban(self, *args):
+
+        self.select_transfers()
+
+        for user in self.selected_users:
+            self.frame.np.network_filter.ban_user(user)
+
+    def on_file_search(self, *args):
+
+        transfer = next(iter(self.selected_transfers), None)
+
+        if not transfer:
+            return
+
+        self.frame.SearchEntry.set_text(transfer.filename.rsplit("\\", 1)[1])
+        self.frame.change_main_page("search")
+
     def translate_status(self, status):
 
         try:
@@ -261,8 +296,8 @@ class TransferList:
             self.update_specific(transfer)
 
         elif self.list is not None:
-            for i in self.list:
-                self.update_specific(i)
+            for transfer in reversed(self.list):
+                self.update_specific(transfer)
 
         if forceupdate or finished or \
                 (curtime - self.last_ui_update) > 1:
@@ -275,27 +310,25 @@ class TransferList:
     def update_parent_rows(self, only_remove=False):
 
         # Remove empty parent rows
-        for (path, pathiter) in list(self.paths.items()):
+        for path, pathiter in list(self.paths.items()):
             if not self.transfersmodel.iter_has_child(pathiter):
                 self.transfersmodel.remove(pathiter)
                 del self.paths[path]
+
             elif not only_remove:
                 self.update_parent_row(pathiter)
 
-        for (username, useriter) in list(self.users.items()):
-            if useriter != 0:
+        for username, useriter in list(self.users.items()):
+            if isinstance(useriter, Gtk.TreeIter):
                 if not self.transfersmodel.iter_has_child(useriter):
                     self.transfersmodel.remove(useriter)
                     del self.users[username]
+
                 elif not only_remove:
                     self.update_parent_row(useriter)
             else:
                 # No grouping
-
-                for transfer in self.list:
-                    if transfer.user == username:
-                        break
-                else:
+                if not self.users[username]:
                     del self.users[username]
 
         self.frame.update_bandwidth()
@@ -484,7 +517,8 @@ class TransferList:
                             empty_int,
                             empty_int,
                             filecount,
-                            empty_int
+                            empty_int,
+                            lambda: None
                         ]
                     )
 
@@ -520,18 +554,16 @@ class TransferList:
                                 empty_int,
                                 empty_int,
                                 filecount,
-                                empty_int
+                                empty_int,
+                                lambda: None
                             ]
                         )
 
                     parent = self.paths[user_path]
             else:
                 # No grouping
-
-                if user not in self.users:
-                    # Insert dummy value. We use this list to get the total number of users
-                    self.users[user] = 0
-
+                # We use this list to get the total number of users
+                self.users.setdefault(user, set()).add(transfer)
                 parent = None
 
             # Add a new transfer
@@ -561,7 +593,8 @@ class TransferList:
                     GObject.Value(GObject.TYPE_UINT64, speed),
                     GObject.Value(GObject.TYPE_UINT64, elapsed),
                     GObject.Value(GObject.TYPE_UINT64, filecount),
-                    GObject.Value(GObject.TYPE_UINT64, place)
+                    GObject.Value(GObject.TYPE_UINT64, place),
+                    transfer
                 )
             )
             transfer.iter = iterator
@@ -572,7 +605,6 @@ class TransferList:
 
                 if self.tree_users == "folder_grouping":
                     # Group by folder, we need the user path to expand it
-
                     user_path = self.transfersmodel.get_path(self.users[user])
                 else:
                     user_path = None
@@ -585,19 +617,25 @@ class TransferList:
 
     def abort_transfers(self, clear=False):
 
-        for i in self.selected_transfers:
-            if i.status != "Finished":
-                self.frame.np.transfers.abort_transfer(i)
+        for transfer in self.selected_transfers:
+            if transfer.status != "Finished":
+                self.frame.np.transfers.abort_transfer(transfer, send_fail_message=True)
 
                 if not clear:
-                    i.status = "Aborted"
-                    self.update(i)
+                    transfer.status = "Aborted"
+                    self.update(transfer)
 
             if clear:
-                self.remove_specific(i)
+                self.remove_specific(transfer)
 
     def remove_specific(self, transfer, cleartreeviewonly=False):
 
+        user = transfer.user
+
+        if user in self.users and not isinstance(self.users[user], Gtk.TreeIter):
+            # No grouping
+            self.users[user].discard(transfer)
+
         if transfer in self.frame.np.transfers.transfer_request_times:
             del self.frame.np.transfers.transfer_request_times[transfer]
 
@@ -611,12 +649,10 @@ class TransferList:
 
     def clear_transfers(self, status):
 
-        for i in self.list[:]:
-            if i.status in status:
-                if i.status == "Queued":
-                    self.frame.np.transfers.abort_transfer(i)
-
-                self.remove_specific(i)
+        for transfer in self.list.copy():
+            if transfer.status in status:
+                self.frame.np.transfers.abort_transfer(transfer, send_fail_message=True)
+                self.remove_specific(transfer)
 
     def clear(self):
 
@@ -627,8 +663,46 @@ class TransferList:
         self.transfersmodel.clear()
 
         if self.list is not None:
-            for i in self.list:
-                i.iter = None
+            for transfer in self.list:
+                transfer.iter = None
+
+    def double_click(self, event):
+
+        self.select_transfers()
+        dc = config.sections["transfers"]["%s_doubleclick" % self.type]
+
+        if dc == 1:  # Send to player
+            self.on_play_files()
+        elif dc == 2:  # File manager
+            self.on_open_directory()
+        elif dc == 3:  # Search
+            self.on_file_search()
+        elif dc == 4:  # Abort
+            self.abort_transfers()
+        elif dc == 5:  # Clear
+            self.abort_transfers(clear=True)
+        elif dc == 6:  # Retry
+            self.retry_transfers()
+
+    def populate_popup_menu_users(self):
+
+        self.popup_menu_users.clear()
+
+        if not self.selected_users:
+            return
+
+        for user in self.selected_users:
+            popup = PopupMenu(self.frame)
+            popup.setup_user_menu(user)
+            popup.setup(
+                ("", None),
+                ("#" + _("Select User's Transfers"), self.on_select_user_transfers, user)
+            )
+
+            popup.toggle_user_items()
+            self.popup_menu_users.setup(
+                (">" + user, popup)
+            )
 
     def expand(self, transfer_path, user_path):
 
@@ -642,7 +716,7 @@ class TransferList:
 
     def on_expand_tree(self, widget):
 
-        expand_button_icon = self.frame.__dict__["Expand%ssImage" % self.type.title()]
+        expand_button_icon = getattr(self.frame, "Expand%ssImage" % self.type.title())
         expanded = widget.get_active()
 
         if expanded:
@@ -652,14 +726,14 @@ class TransferList:
             collapse_treeview(self.widget, self.tree_users)
             expand_button_icon.set_from_icon_name("go-down-symbolic", Gtk.IconSize.BUTTON)
 
-        self.frame.np.config.sections["transfers"]["%ssexpanded" % self.type] = expanded
-        self.frame.np.config.write_configuration()
+        config.sections["transfers"]["%ssexpanded" % self.type] = expanded
+        config.write_configuration()
 
     def on_toggle_tree(self, widget):
 
         active = widget.get_active()
 
-        self.frame.np.config.sections["transfers"]["group%ss" % self.type] = active
+        config.sections["transfers"]["group%ss" % self.type] = active
         self.widget.set_show_expanders(active)
         self.expand_button.set_visible(active)
 
@@ -669,11 +743,49 @@ class TransferList:
     def on_tooltip(self, widget, x, y, keyboard_mode, tooltip):
         return show_file_path_tooltip(widget, x, y, tooltip, 10)
 
+    def on_popup_menu(self, *args):
+
+        self.select_transfers()
+        num_selected_transfers = len(self.selected_transfers)
+
+        actions = self.popup_menu.get_actions()
+        users = len(self.selected_users) > 0
+        files = num_selected_transfers > 0
+
+        actions[_("User(s)")].set_enabled(users)  # Users Menu
+        self.populate_popup_menu_users()
+
+        if files:
+            act = True
+        else:
+            # Disable options
+            # Send to player, File manager, file properties, Copy File Path, Copy URL, Copy Folder URL, Search filename
+            act = False
+
+        for i in (_("Send to _Player"), _("_Open Folder"), _("File P_roperties"),
+                  _("Copy _File Path"), _("Copy _URL"), _("Copy Folder URL"), _("_Search")):
+            actions[i].set_enabled(act)
+
+        if not users or not files:
+            # Disable options
+            # Retry, Abort, Clear
+            act = False
+        else:
+            act = True
+
+        for i in (_("_Retry"), _("Abor_t"), _("_Clear")):
+            actions[i].set_enabled(act)
+
+        self.popup_menu.set_num_selected_files(num_selected_transfers)
+
+        self.popup_menu.popup()
+        return True
+
     def on_list_clicked(self, widget, event):
 
         if triggers_context_menu(event):
             set_treeview_selected_row(widget, event)
-            return self.on_popup_menu(widget)
+            return self.on_popup_menu()
 
         if event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
             self.double_click(event)
@@ -681,41 +793,12 @@ class TransferList:
 
         return False
 
-    def on_popup_menu_users(self, widget):
-
-        self.popup_menu_users.clear()
-
-        if self.selected_users:
-
-            items = []
-
-            for user in self.selected_users:
-
-                popup = PopupMenu(self.frame, False)
-                popup.setup_user_menu(user)
-                popup.append_item(("", None))
-                popup.append_item(("#" + _("Select User's Transfers"), self.on_select_user_transfers))
-
-                items.append((1, user, popup, self.on_popup_menu_user, popup))
-
-            self.popup_menu_users.setup(*items)
-
-        return True
-
-    def on_popup_menu_user(self, widget, popup=None):
-
-        if popup is None:
-            return
-
-        popup.toggle_user_items()
-        return True
-
-    def on_select_user_transfers(self, widget):
+    def on_select_user_transfers(self, *args):
 
         if not self.selected_users:
             return
 
-        selected_user = widget.get_parent().user
+        selected_user = args[-1]
 
         sel = self.widget.get_selection()
         fmodel = self.widget.get_model()
@@ -733,17 +816,17 @@ class TransferList:
         self.select_transfers()
 
         if keycode in keyval_to_hardware_keycode(Gdk.KEY_t):
-            self.on_abort_transfer(widget)
+            self.abort_transfers()
 
         elif keycode in keyval_to_hardware_keycode(Gdk.KEY_r):
-            self.on_retry_transfer(widget)
+            self.retry_transfers()
 
         elif event.get_state() & Gdk.ModifierType.CONTROL_MASK and \
                 keycode in keyval_to_hardware_keycode(Gdk.KEY_c):
-            self.on_copy_file_path(widget)
+            self.on_copy_file_path()
 
         elif keycode in keyval_to_hardware_keycode(Gdk.KEY_Delete):
-            self.on_abort_transfer(widget, clear=True)
+            self.abort_transfers(clear=True)
 
         else:
             # No key match, continue event
@@ -752,56 +835,96 @@ class TransferList:
         widget.stop_emission_by_name("key_press_event")
         return True
 
-    def on_copy_file_path(self, widget):
+    def on_file_properties(self, *args):
 
-        if not self.selected_transfers:
+        if not self.frame.np.transfers:
             return
 
-        i = next(iter(self.selected_transfers))
-        text = self.transfersmodel.get_value(i.iter, 10)
+        data = []
+        model, paths = self.widget.get_selection().get_selected_rows()
 
-        self.frame.clip.set_text(text, -1)
+        for path in paths:
+            iterator = model.get_iter(path)
+            transfer = model.get_value(iterator, 18)
 
-    def on_copy_url(self, widget):
-        i = next(iter(self.selected_transfers))
-        self.frame.set_clipboard_url(i.user, i.filename)
+            if not isinstance(transfer, Transfer):
+                continue
 
-    def on_copy_dir_url(self, widget):
+            user = model.get_value(iterator, 0)
+            filename = model.get_value(iterator, 2)
+            fullname = model.get_value(iterator, 10)
+            size = speed = length = queue = immediate = num = country = bitratestr = ""
 
-        i = next(iter(self.selected_transfers))
-        path = "\\".join(i.filename.split("\\")[:-1])
+            size = str(human_size(transfer.size))
 
-        if path[:-1] != "/":
-            path += "/"
+            if transfer.speed:
+                speed = str(human_speed(transfer.speed))
 
-        self.frame.set_clipboard_url(i.user, path)
+            bitratestr = str(transfer.bitrate)
+            length = str(transfer.length)
 
-    def on_retry_transfer(self, widget):
+            directory = fullname.rsplit("\\", 1)[0]
+
+            data.append({
+                "user": user,
+                "fn": fullname,
+                "position": num,
+                "filename": filename,
+                "directory": directory,
+                "size": size,
+                "speed": speed,
+                "queue": queue,
+                "immediate": immediate,
+                "bitrate": bitratestr,
+                "length": length,
+                "country": country
+            })
+
+        if paths:
+            FileProperties(self.frame, data).show()
+
+    def on_copy_file_path(self, *args):
+
+        transfer = next(iter(self.selected_transfers), None)
+
+        if transfer:
+            self.frame.clipboard.set_text(transfer.filename, -1)
+
+    def on_copy_url(self, *args):
+
+        transfer = next(iter(self.selected_transfers), None)
+
+        if transfer:
+            copy_file_url(transfer.user, transfer.filename, self.frame.clipboard)
+
+    def on_copy_dir_url(self, *args):
+
+        transfer = next(iter(self.selected_transfers), None)
+
+        if transfer:
+            copy_file_url(transfer.user, transfer.filename.rsplit('\\', 1)[0] + '\\', self.frame.clipboard)
+
+    def on_retry_transfer(self, *args):
         self.select_transfers()
         self.retry_transfers()
 
-    def on_abort_transfer(self, widget, clear=False):
+    def on_abort_transfer(self, *args):
         self.select_transfers()
-        self.abort_transfers(clear)
+        self.abort_transfers()
 
-    def on_clear_transfer(self, widget):
+    def on_clear_transfer(self, *args):
         self.select_transfers()
         self.abort_transfers(clear=True)
 
-    def on_clear_response(self, dialog, response, data=None):
-        if response == Gtk.ResponseType.OK:
-            self.clear_transfers(["Queued"])
+    def on_clear_response(self, dialog, response_id, data):
 
         dialog.destroy()
 
-    def on_clear_queued(self, widget):
+        if response_id == Gtk.ResponseType.OK:
+            self.clear_transfers(["Queued"])
+
+    def on_clear_queued(self, *args):
         self.clear_transfers(["Queued"])
 
-    def on_clear_finished(self, widget):
+    def on_clear_finished(self, *args):
         self.clear_transfers(["Finished"])
-
-    def on_clear_filtered(self, widget):
-        self.clear_transfers(["Filtered"])
-
-    def on_clear_paused(self, widget):
-        self.clear_transfers(["Paused"])
