@@ -24,22 +24,24 @@ import os
 import time
 
 from gi.repository import GdkPixbuf
+from gi.repository import GLib
 from gi.repository import Gtk
 
 from pynicotine import slskmessages
-from pynicotine.gtkgui.dialogs import save_file
+from pynicotine.config import config
 from pynicotine.gtkgui.utils import append_line
-from pynicotine.gtkgui.utils import humanize
-from pynicotine.gtkgui.utils import human_speed
-from pynicotine.gtkgui.utils import IconNotebook
-from pynicotine.gtkgui.utils import InfoBar
-from pynicotine.gtkgui.utils import initialise_columns
 from pynicotine.gtkgui.utils import load_ui_elements
-from pynicotine.gtkgui.utils import PopupMenu
-from pynicotine.gtkgui.utils import set_treeview_selected_row
 from pynicotine.gtkgui.utils import triggers_context_menu
-from pynicotine.gtkgui.utils import update_widget_visuals
+from pynicotine.gtkgui.widgets.filechooser import save_file
+from pynicotine.gtkgui.widgets.iconnotebook import IconNotebook
+from pynicotine.gtkgui.widgets.infobar import InfoBar
+from pynicotine.gtkgui.widgets.popupmenu import PopupMenu
+from pynicotine.gtkgui.widgets.theme import update_widget_visuals
+from pynicotine.gtkgui.widgets.treeview import initialise_columns
+from pynicotine.gtkgui.widgets.treeview import set_treeview_selected_row
 from pynicotine.logfacility import log
+from pynicotine.utils import humanize
+from pynicotine.utils import human_speed
 
 
 # User Info and User Browse Notebooks
@@ -49,16 +51,14 @@ class UserTabs(IconNotebook):
 
         self.frame = frame
 
-        ui = frame.np.config.sections["ui"]
-
         IconNotebook.__init__(
             self,
             self.frame.images,
-            angle=ui["labelinfo"],
-            tabclosers=ui["tabclosers"],
-            show_hilite_image=self.frame.np.config.sections["notifications"]["notification_tab_icons"],
-            reorderable=ui["tab_reorderable"],
-            show_status_image=ui["tab_status_icons"],
+            angle=config.sections["ui"]["labelinfo"],
+            tabclosers=config.sections["ui"]["tabclosers"],
+            show_hilite_image=config.sections["notifications"]["notification_tab_icons"],
+            reorderable=config.sections["ui"]["tab_reorderable"],
+            show_status_image=config.sections["ui"]["tab_status_icons"],
             notebookraw=notebookraw
         )
 
@@ -106,8 +106,14 @@ class UserTabs(IconNotebook):
             self.users[user].show_connection_error()
 
     def save_columns(self):
-        for user in self.users:
-            self.users[user].save_columns()
+        """ Save the treeview state of the currently selected tab """
+
+        current_page = self.get_nth_page(self.get_current_page())
+
+        for tab in self.users.values():
+            if tab.Main == current_page:
+                tab.save_columns()
+                break
 
     def get_user_stats(self, msg):
 
@@ -149,19 +155,15 @@ class UserTabs(IconNotebook):
         for i in self.users.values():
             i.update_visuals()
 
-    def tab_popup(self, user):
-
-        if user in self.users:
-            return self.users[user].tab_popup(user)
-
     def on_tab_popup(self, widget, page):
 
         username = self.get_page_owner(page, self.users)
-        menu = self.tab_popup(username)
 
-        if menu is None:
+        if username not in self.users:
             return False
 
+        menu = self.users[username].user_popup
+        menu.toggle_user_items()
         menu.popup()
         return True
 
@@ -208,6 +210,7 @@ class UserInfo:
         self.hates_store = Gtk.ListStore(str)
         self.Hates.set_model(self.hates_store)
 
+        self.hate_column_numbers = list(range(self.hates_store.get_n_columns()))
         cols = initialise_columns(
             None,
             self.Hates,
@@ -220,6 +223,7 @@ class UserInfo:
         self.likes_store = Gtk.ListStore(str)
         self.Likes.set_model(self.likes_store)
 
+        self.like_column_numbers = list(range(self.likes_store.get_n_columns()))
         cols = initialise_columns(
             None,
             self.Likes,
@@ -234,27 +238,19 @@ class UserInfo:
         self.update_visuals()
 
         self.user_popup = popup = PopupMenu(self.frame)
-        popup.setup_user_menu(user)
-        popup.get_items()[_("Show User I_nfo")].set_visible(False)
-
-        popup.append_item(("", None))
-        popup.append_item(("#" + _("Close All Tabs"), self.on_close_all_tabs))
-        popup.append_item(("#" + _("_Close Tab"), self.on_close))
-
-        self.likes_popup_menu = popup = PopupMenu(self.frame)
+        popup.setup_user_menu(user, page="userinfo")
         popup.setup(
-            ("$" + _("I _Like This"), self.frame.interests.on_like_recommendation),
-            ("$" + _("I _Dislike This"), self.frame.interests.on_dislike_recommendation),
             ("", None),
-            ("#" + _("_Search For Item"), self.frame.interests.on_recommend_search)
+            ("#" + _("Close All Tabs"), self.on_close_all_tabs),
+            ("#" + _("_Close Tab"), self.on_close)
         )
 
-        self.hates_popup_menu = popup = PopupMenu(self.frame)
+        self.interest_popup_menu = popup = PopupMenu(self.frame)
         popup.setup(
-            ("$" + _("I _Like This"), self.frame.interests.on_like_recommendation),
-            ("$" + _("I _Dislike This"), self.frame.interests.on_dislike_recommendation),
+            ("$" + _("I _Like This"), self.on_like_recommendation),
+            ("$" + _("I _Dislike This"), self.on_dislike_recommendation),
             ("", None),
-            ("#" + _("_Search For Item"), self.frame.interests.on_recommend_search)
+            ("#" + _("_Search For Item"), self.on_interest_recommend_search)
         )
 
         self.image_menu = popup = PopupMenu(self.frame)
@@ -266,64 +262,9 @@ class UserInfo:
             ("#" + _("Save Picture"), self.on_save_picture)
         )
 
-    def get_selected_like_item(self, treeview):
-
-        model, iterator = treeview.get_selection().get_selected()
-
-        if iterator is None:
-            return None
-
-        return model.get_value(iterator, 0)
-
-    def on_likes_list_clicked(self, widget, event):
-
-        if triggers_context_menu(event):
-            set_treeview_selected_row(widget, event)
-            return self.on_popup_likes_menu(widget)
-
-        return False
-
-    def on_popup_likes_menu(self, widget):
-
-        item = self.get_selected_like_item(widget)
-        if item is None:
-            return False
-
-        self.likes_popup_menu.set_user(item)
-
-        items = self.likes_popup_menu.get_items()
-        items[_("I _Like This")].set_active(item in self.frame.np.config.sections["interests"]["likes"])
-        items[_("I _Dislike This")].set_active(item in self.frame.np.config.sections["interests"]["dislikes"])
-
-        self.likes_popup_menu.popup()
-        return True
-
-    def on_hates_list_clicked(self, widget, event):
-
-        if triggers_context_menu(event):
-            set_treeview_selected_row(widget, event)
-            return self.on_popup_hates_menu(widget)
-
-        return False
-
-    def on_popup_hates_menu(self, widget):
-
-        item = self.get_selected_like_item(widget)
-        if item is None:
-            return False
-
-        self.hates_popup_menu.set_user(item)
-
-        items = self.hates_popup_menu.get_items()
-        items[_("I _Like This")].set_active(item in self.frame.np.config.sections["interests"]["likes"])
-        items[_("I _Dislike This")].set_active(item in self.frame.np.config.sections["interests"]["dislikes"])
-
-        self.hates_popup_menu.popup()
-        return True
-
     def update_visuals(self):
 
-        for widget in self.__dict__.values():
+        for widget in list(self.__dict__.values()):
             update_widget_visuals(widget)
 
     def show_interests(self, likes, hates):
@@ -332,10 +273,10 @@ class UserInfo:
         self.hates_store.clear()
 
         for like in likes:
-            self.likes_store.append([like])
+            self.likes_store.insert_with_valuesv(-1, self.like_column_numbers, [like])
 
         for hate in hates:
-            self.hates_store.append([hate])
+            self.hates_store.insert_with_valuesv(-1, self.hate_column_numbers, [hate])
 
     def save_columns(self):
         # Unused
@@ -432,78 +373,113 @@ class UserInfo:
 
         self.progressbar.set_fraction(fraction)
 
-    def tab_popup(self, user):
-        self.user_popup.toggle_user_items()
-        return self.user_popup
-
     """ Events """
 
-    def on_send_message(self, widget):
+    def on_popup_interest_menu(self, widget):
+
+        model, iterator = widget.get_selection().get_selected()
+
+        if iterator is None:
+            return False
+
+        item = model.get_value(iterator, 0)
+
+        if item is None:
+            return False
+
+        self.interest_popup_menu.set_user(item)
+
+        actions = self.interest_popup_menu.get_actions()
+        actions[_("I _Like This")].set_state(
+            GLib.Variant.new_boolean(item in config.sections["interests"]["likes"])
+        )
+        actions[_("I _Dislike This")].set_state(
+            GLib.Variant.new_boolean(item in config.sections["interests"]["dislikes"])
+        )
+
+        self.interest_popup_menu.popup()
+        return True
+
+    def on_like_recommendation(self, action, state):
+        self.frame.interests.on_like_recommendation(action, state, self.interest_popup_menu.get_user())
+
+    def on_dislike_recommendation(self, action, state):
+        self.frame.interests.on_dislike_recommendation(action, state, self.interest_popup_menu.get_user())
+
+    def on_interest_recommend_search(self, *args):
+        self.frame.interests.recommend_search(self.interest_popup_menu.get_user())
+
+    def on_interest_list_clicked(self, widget, event):
+
+        if triggers_context_menu(event):
+            set_treeview_selected_row(widget, event)
+            return self.on_popup_interest_menu(widget)
+
+        return False
+
+    def on_send_message(self, *args):
         self.frame.privatechats.send_message(self.user, show_user=True)
         self.frame.change_main_page("private")
 
-    def on_show_ip_address(self, widget):
-
+    def on_show_ip_address(self, *args):
         self.frame.np.ip_requested.add(self.user)
         self.frame.np.queue.append(slskmessages.GetPeerAddress(self.user))
 
-    def on_refresh(self, widget):
+    def on_refresh(self, *args):
         self.info_bar.set_visible(False)
         self.progressbar.set_fraction(0.0)
         self.frame.local_user_info_request(self.user)
 
-    def on_browse_user(self, widget):
+    def on_browse_user(self, *args):
         self.frame.browse_user(self.user)
 
-    def on_add_to_list(self, widget):
+    def on_add_to_list(self, *args):
         self.frame.np.userlist.add_to_list(self.user)
 
-    def on_ban_user(self, widget):
+    def on_ban_user(self, *args):
         self.frame.np.network_filter.ban_user(self.user)
 
-    def on_ignore_user(self, widget):
+    def on_ignore_user(self, *args):
         self.frame.np.network_filter.ignore_user(self.user)
 
-    def on_save_picture(self, widget):
+    def on_save_picture_response(self, selected, data):
+
+        if not os.path.exists(selected):
+            self.image_pixbuf.savev(selected, "jpeg", ["quality"], ["100"])
+            log.add(_("Picture saved to %s"), selected)
+        else:
+            log.add(_("Picture not saved, %s already exists."), selected)
+
+    def on_save_picture(self, *args):
 
         if self.image is None or self.image_pixbuf is None:
             return
 
-        response = save_file(
-            self.frame.MainWindow,
-            self.frame.np.config.sections["transfers"]["downloaddir"],
-            "%s %s.jpg" % (self.user, time.strftime("%Y-%m-%d %H_%M_%S")),
+        save_file(
+            parent=self.frame.MainWindow,
+            callback=self.on_save_picture_response,
+            initialdir=config.sections["transfers"]["downloaddir"],
+            initialfile="%s %s.jpg" % (self.user, time.strftime("%Y-%m-%d %H_%M_%S")),
             title="Save as..."
         )
 
-        if not response:
-            return
-
-        pathname = response[0]
-
-        if not os.path.exists(pathname):
-            self.image_pixbuf.savev(pathname, "jpeg", ["quality"], ["100"])
-            log.add(_("Picture saved to %s"), pathname)
-        else:
-            log.add(_("Picture not saved, %s already exists."), pathname)
-
     def on_image_click(self, widget, event):
 
         if triggers_context_menu(event):
-            return self.on_image_popup_menu(widget)
+            return self.on_image_popup_menu()
 
         return False
 
-    def on_image_popup_menu(self, widget):
+    def on_image_popup_menu(self, *args):
 
         act = True
 
         if self.image is None or self.image_pixbuf is None:
             act = False
 
-        items = self.image_menu.get_items()
-        for (item_id, item) in items.items():
-            item.set_sensitive(act)
+        actions = self.image_menu.get_actions()
+        for (action_id, action) in actions.items():
+            action.set_enabled(act)
 
         self.image_menu.popup()
         return True
@@ -517,10 +493,10 @@ class UserInfo:
 
         return True  # Don't scroll the Gtk.ScrolledWindow
 
-    def make_zoom_normal(self, widget):
+    def make_zoom_normal(self, *args):
         self.make_zoom_in(zoom=True)
 
-    def make_zoom_in(self, widget=None, zoom=None):
+    def make_zoom_in(self, *args, zoom=None):
 
         def calc_zoom_in(a):
             return a + a * self.actual_zoom / 100 + a * self.zoom_factor / 100
@@ -545,7 +521,7 @@ class UserInfo:
 
         gc.collect()
 
-    def make_zoom_out(self, widget=None):
+    def make_zoom_out(self, *args):
 
         def calc_zoom_out(a):
             return a + a * self.actual_zoom / 100 - a * self.zoom_factor / 100
@@ -571,9 +547,9 @@ class UserInfo:
 
         gc.collect()
 
-    def on_close(self, widget):
+    def on_close(self, *args):
         del self.userinfos.users[self.user]
         self.userinfos.remove_page(self.Main)
 
-    def on_close_all_tabs(self, widget):
+    def on_close_all_tabs(self, *args):
         self.userinfos.remove_all_pages()
