@@ -30,19 +30,20 @@ from gi.repository import GObject
 from gi.repository import Gtk
 
 from pynicotine import slskmessages
-from pynicotine.gtkgui.dialogs import entry_dialog
-from pynicotine.gtkgui.utils import humanize
-from pynicotine.gtkgui.utils import human_speed
-from pynicotine.gtkgui.utils import initialise_columns
+from pynicotine.config import config
 from pynicotine.gtkgui.utils import load_ui_elements
-from pynicotine.gtkgui.utils import PopupMenu
-from pynicotine.gtkgui.utils import save_columns
-from pynicotine.gtkgui.utils import set_treeview_selected_row
-from pynicotine.gtkgui.utils import show_country_tooltip
-from pynicotine.gtkgui.utils import show_user_status_tooltip
 from pynicotine.gtkgui.utils import triggers_context_menu
-from pynicotine.gtkgui.utils import update_widget_visuals
+from pynicotine.gtkgui.widgets.messagedialogs import entry_dialog
+from pynicotine.gtkgui.widgets.popupmenu import PopupMenu
+from pynicotine.gtkgui.widgets.theme import update_widget_visuals
+from pynicotine.gtkgui.widgets.treeview import initialise_columns
+from pynicotine.gtkgui.widgets.treeview import save_columns
+from pynicotine.gtkgui.widgets.treeview import set_treeview_selected_row
+from pynicotine.gtkgui.widgets.treeview import show_country_tooltip
+from pynicotine.gtkgui.widgets.treeview import show_user_status_tooltip
 from pynicotine.logfacility import log
+from pynicotine.utils import humanize
+from pynicotine.utils import human_speed
 
 
 class UserList:
@@ -51,12 +52,12 @@ class UserList:
 
         # Build the window
         self.frame = frame
-        config = self.frame.np.config.sections
 
         load_ui_elements(self, os.path.join(self.frame.gui_dir, "ui", "buddylist.ui"))
 
         """ Columns """
 
+        self.user_iterators = {}
         self.usersmodel = Gtk.ListStore(
             GObject.TYPE_OBJECT,  # (0)  status icon
             GObject.TYPE_OBJECT,  # (1)  flag
@@ -105,9 +106,8 @@ class UserList:
         cols["status"].get_widget().hide()
         cols["country"].get_widget().hide()
 
-        if config["columns"]["hideflags"]:
-            cols["country"].set_visible(0)
-            config["columns"]["buddy_list"]["country"]["visible"] = 0
+        if config.sections["columns"]["hideflags"]:
+            cols["country"].set_visible(False)
 
         for render in cols["trusted"].get_cells():
             render.connect('toggled', self.cell_toggle_callback, self.UserListTree, 5)
@@ -125,7 +125,7 @@ class UserList:
 
         """ Buddy list """
 
-        for user in self.frame.np.config.sections["server"]["userlist"]:
+        for user in config.sections["server"]["userlist"]:
             try:
                 username, comment, notify, privileged, trusted, last_seen, country = user
             except ValueError:
@@ -138,15 +138,16 @@ class UserList:
                 last_seen = _("Never seen")
                 time_from_epoch = 0
 
+            username = str(username)
             row = [
-                self.frame.get_status_image(0),
-                self.frame.get_flag_image(country),
-                str(username),
+                GObject.Value(GObject.TYPE_OBJECT, self.frame.get_status_image(0)),
+                GObject.Value(GObject.TYPE_OBJECT, self.frame.get_flag_image(country)),
+                username,
                 "",
                 "",
-                trusted,
-                notify,
-                privileged,
+                bool(trusted),
+                bool(notify),
+                bool(privileged),
                 str(last_seen),
                 str(comment),
                 0,
@@ -156,7 +157,7 @@ class UserList:
                 str(country)
             ]
 
-            self.usersmodel.insert(0, row)
+            self.user_iterators[username] = self.usersmodel.insert_with_valuesv(0, self.column_numbers, row)
 
         self.usersmodel.set_sort_column_id(2, Gtk.SortType.ASCENDING)
 
@@ -168,19 +169,20 @@ class UserList:
 
         """ Popup """
 
-        self.popup_menu_private_rooms = PopupMenu(self.frame, False)
-        self.popup_menu = popup = PopupMenu(frame)
-        popup.setup_user_menu()
-        popup.get_items()[_("_Add User To List")].set_visible(False)
+        self.popup_menu_private_rooms = PopupMenu(self.frame)
 
-        popup.append_item(("", None))
-        popup.append_item(("$" + _("_Online Notify"), self.on_notify))
-        popup.append_item(("$" + _("_Privileged"), self.on_privileged))
-        popup.append_item(("$" + _("_Trusted"), self.on_trusted))
-        popup.append_item(("", None))
-        popup.append_item((1, _("Private Rooms"), self.popup_menu_private_rooms, popup.on_private_rooms, self.popup_menu_private_rooms))
-        popup.append_item(("#" + _("Edit _Comments"), self.on_edit_comments))
-        popup.append_item(("#" + _("_Remove"), self.on_remove_user))
+        self.popup_menu = popup = PopupMenu(frame)
+        popup.setup_user_menu(page="userlist")
+        popup.setup(
+            ("", None),
+            ("$" + _("_Online Notify"), self.on_notify),
+            ("$" + _("_Privileged"), self.on_privileged),
+            ("$" + _("_Trusted"), self.on_trusted),
+            ("", None),
+            (">" + _("Private Rooms"), self.popup_menu_private_rooms),
+            ("#" + _("Edit _Comments"), self.on_edit_comments),
+            ("#" + _("_Remove"), self.on_remove_user)
+        )
 
         self.update_visuals()
 
@@ -190,7 +192,7 @@ class UserList:
             widget.remove_all()
             widget.append_text("")
 
-            for user in self.frame.np.config.sections["server"]["userlist"]:
+            for user in config.sections["server"]["userlist"]:
                 widget.append_text(str(user[0]))
 
     def on_tooltip(self, widget, x, y, keyboard_mode, tooltip):
@@ -216,31 +218,39 @@ class UserList:
 
     def update_visuals(self):
 
-        for widget in self.__dict__.values():
+        for widget in list(self.__dict__.values()):
             update_widget_visuals(widget)
 
-    def on_settings_ban_ignore(self, widget):
-        self.frame.on_settings_ban_ignore(widget)
+    def on_settings_ban_ignore(self, *args):
+        self.frame.on_settings_ban_ignore()
 
     def cell_toggle_callback(self, widget, index, treeview, pos):
 
-        iterator = self.usersmodel.get_iter(index)
-        value = self.usersmodel.get_value(iterator, pos)
+        store = treeview.get_model()
+        iterator = store.get_iter(index)
 
+        value = self.usersmodel.get_value(iterator, pos)
         self.usersmodel.set_value(iterator, pos, not value)
 
         self.save_user_list()
 
     def cell_edited_callback(self, widget, index, value, treeview, pos):
 
+        if pos != 9:
+            return
+
         store = treeview.get_model()
         iterator = store.get_iter(index)
 
-        if pos == 9:
-            self.set_comment(iterator, store, value)
+        self.set_comment(iterator, store, value)
 
     def set_last_seen(self, user, online=False):
 
+        iterator = self.user_iterators.get(user)
+
+        if iterator is None:
+            return
+
         last_seen = ""
         time_from_epoch = 2147483647  # Gtk only allows range -2147483648 to 2147483647 in set()
 
@@ -248,26 +258,16 @@ class UserList:
             last_seen = time.strftime("%m/%d/%Y %H:%M:%S")
             time_from_epoch = time.mktime(time.strptime(last_seen, "%m/%d/%Y %H:%M:%S"))
 
-        for i in self.usersmodel:
-            if i[2] == user:
-                self.usersmodel.set_value(i.iter, 8, last_seen)
-                self.usersmodel.set_value(i.iter, 13, int(time_from_epoch))
-                break
+        self.usersmodel.set_value(iterator, 8, last_seen)
+        self.usersmodel.set_value(iterator, 13, int(time_from_epoch))
 
         if not online:
             self.save_user_list()
 
     def set_comment(self, iterator, store, comments=None):
 
-        user = store.get_value(iterator, 2)
-
         if comments is not None:
-
-            for i in self.usersmodel:
-                if i[2] == user:
-                    self.usersmodel.set_value(iterator, 9, comments)
-                    break
-
+            store.set_value(iterator, 9, comments)
             self.save_user_list()
 
     def conn_close(self):
@@ -282,8 +282,8 @@ class UserList:
             self.usersmodel.set_value(iterator, 11, 0)
             self.usersmodel.set_value(iterator, 12, 0)
 
-            if self.usersmodel.get(iterator, 8)[0] == "":
-                user = i[2]
+            if not self.usersmodel.get_value(iterator, 8):
+                user = self.usersmodel.get_value(iterator, 2)
                 self.set_last_seen(user)
 
     def get_selected_username(self, treeview):
@@ -335,29 +335,22 @@ class UserList:
 
         self.popup_menu.set_user(username)
         self.popup_menu.toggle_user_items()
+        self.popup_menu.populate_private_rooms(self.popup_menu_private_rooms)
 
-        items = self.popup_menu.get_items()
+        actions = self.popup_menu.get_actions()
 
-        items[_("Private Rooms")].set_sensitive(
+        actions[_("Private Rooms")].set_enabled(
             status or
-            self.popup_menu.user != self.frame.np.config.sections["server"]["login"]
+            self.popup_menu.user != config.sections["server"]["login"]
         )
 
-        items[_("_Online Notify")].set_active(notify)
-        items[_("_Privileged")].set_active(privileged)
-        items[_("_Trusted")].set_active(trusted)
+        actions[_("_Online Notify")].set_state(GLib.Variant.new_boolean(notify))
+        actions[_("_Privileged")].set_state(GLib.Variant.new_boolean(privileged))
+        actions[_("_Trusted")].set_state(GLib.Variant.new_boolean(trusted))
 
         self.popup_menu.popup()
         return True
 
-    def get_iter(self, user):
-
-        for i in self.usersmodel:
-            if i[2] == user:
-                return i.iter
-
-        return None
-
     def get_user_status(self, msg):
 
         status = msg.status
@@ -367,7 +360,7 @@ class UserList:
             return
 
         user = msg.user
-        iterator = self.get_iter(user)
+        iterator = self.user_iterators.get(user)
 
         if iterator is None:
             return
@@ -394,13 +387,14 @@ class UserList:
 
         if status:  # online
             self.set_last_seen(user, online=True)
-        elif self.usersmodel.get(iterator, 8)[0] == "":  # disconnected
+
+        elif not self.usersmodel.get_value(iterator, 8):  # disconnected
             self.set_last_seen(user)
 
     def get_user_stats(self, msg):
 
         user = msg.user
-        iterator = self.get_iter(user)
+        iterator = self.user_iterators.get(user)
 
         if iterator is None:
             return
@@ -415,11 +409,9 @@ class UserList:
 
     def set_user_flag(self, user, country):
 
-        iterator = self.get_iter(user)
-        if iterator is None:
-            return
+        iterator = self.user_iterators.get(user)
 
-        if user not in (i[2] for i in self.usersmodel):
+        if iterator is None:
             return
 
         self.usersmodel.set_value(iterator, 1, GObject.Value(GObject.TYPE_OBJECT, self.frame.get_flag_image(country)))
@@ -427,13 +419,13 @@ class UserList:
 
     def add_to_list(self, user):
 
-        if user in (i[2] for i in self.usersmodel):
+        if user in self.user_iterators:
             return
 
         empty_int = 0
         empty_str = ""
 
-        self.usersmodel.insert_with_valuesv(
+        self.user_iterators[user] = self.usersmodel.insert_with_valuesv(
             -1, self.column_numbers,
             [
                 GObject.Value(GObject.TYPE_OBJECT, self.frame.get_status_image(0)),
@@ -465,31 +457,20 @@ class UserList:
         for widget in self.buddies_combo_entries:
             widget.append_text(user)
 
-        if self.frame.np.config.sections["words"]["buddies"]:
-            GLib.idle_add(self.frame.chatrooms.update_completions)
-            GLib.idle_add(self.frame.privatechats.update_completions)
+        if config.sections["words"]["buddies"]:
+            self.frame.update_completions()
 
-    def on_edit_comments(self, widget):
+    def remove_from_list(self, user):
 
-        user = self.popup_menu.get_user()
+        if user in self.user_iterators:
+            self.usersmodel.remove(self.user_iterators[user])
+            del self.user_iterators[user]
 
-        for i in self.usersmodel:
-            if i[2] == user:
-                comments = i[9]
-                break
-        else:
-            comments = ""
+        self.save_user_list()
+        self.buddies_combos_fill()
 
-        comments = entry_dialog(self.frame.MainWindow, _("Edit comments") + "...", _("Comments") + ":", comments)
-
-        if comments is not None:
-            for i in self.usersmodel:
-                if i[2] == user:
-                    i[9] = comments
-                    self.usersmodel.set_value(i.iter, 9, comments)
-                    break
-
-            self.save_user_list()
+        if config.sections["words"]["buddies"]:
+            self.frame.update_completions()
 
     def save_user_list(self):
 
@@ -499,59 +480,85 @@ class UserList:
             status_icon, flag, user, hspeed, hfile_count, trusted, notify, privileged, hlast_seen, comments, status, speed, file_count, last_seen, country = i
             user_list.append([user, comments, notify, privileged, trusted, hlast_seen, country])
 
-        self.frame.np.config.sections["server"]["userlist"] = user_list
-        self.frame.np.config.write_configuration()
+        config.sections["server"]["userlist"] = user_list
+        config.write_configuration()
 
     def save_columns(self):
         save_columns("buddy_list", self.UserListTree.get_columns())
 
-    def remove_from_list(self, user):
+    def on_trusted(self, action, state):
 
-        for i in self.usersmodel:
-            if i[2] == user:
-                self.usersmodel.remove(i.iter)
-                break
+        user = self.popup_menu.get_user()
+        iterator = self.user_iterators.get(user)
+
+        if iterator is None:
+            return
+
+        self.usersmodel.set_value(iterator, 5, state)
 
         self.save_user_list()
+        action.set_state(state)
 
-        self.buddies_combos_fill()
+    def on_notify(self, action, state):
 
-        if self.frame.np.config.sections["words"]["buddies"]:
-            GLib.idle_add(self.frame.chatrooms.update_completions)
-            GLib.idle_add(self.frame.privatechats.update_completions)
+        user = self.popup_menu.get_user()
+        iterator = self.user_iterators.get(user)
 
-    def on_remove_user(self, widget):
+        if iterator is None:
+            return
+
+        self.usersmodel.set_value(iterator, 6, state)
+
+        self.save_user_list()
+        action.set_state(state)
+
+    def on_privileged(self, action, state):
+
+        user = self.popup_menu.get_user()
+        iterator = self.user_iterators.get(user)
+
+        if iterator is None:
+            return
+
+        self.usersmodel.set_value(iterator, 7, state)
+
+        self.save_user_list()
+        action.set_state(state)
+
+    def on_edit_comments_response(self, dialog, response_id, user):
+
+        iterator = self.user_iterators.get(user)
+
+        if iterator is None:
+            return
+
+        comments = dialog.get_response_value()
+        dialog.destroy()
+
+        if comments is None:
+            return
+
+        self.usersmodel.set_value(iterator, 9, comments)
+        self.save_user_list()
+
+    def on_edit_comments(self, *args):
+
+        user = self.popup_menu.get_user()
+        iterator = self.user_iterators.get(user)
+
+        if iterator is None:
+            return
+
+        comments = self.usersmodel.get_value(iterator, 9) or ""
+
+        entry_dialog(
+            parent=self.frame.MainWindow,
+            title=_("Edit comments") + "...",
+            message=_("Comments") + ":",
+            callback=self.on_edit_comments_response,
+            callback_data=user,
+            default=comments
+        )
+
+    def on_remove_user(self, *args):
         self.remove_from_list(self.popup_menu.get_user())
-
-    def on_trusted(self, widget):
-
-        user = self.popup_menu.get_user()
-
-        for i in self.usersmodel:
-            if i[2] == user:
-                self.usersmodel.set_value(i.iter, 5, widget.get_active())
-                break
-
-        self.save_user_list()
-
-    def on_notify(self, widget):
-
-        user = self.popup_menu.get_user()
-
-        for i in self.usersmodel:
-            if i[2] == user:
-                self.usersmodel.set_value(i.iter, 6, widget.get_active())
-                break
-
-        self.save_user_list()
-
-    def on_privileged(self, widget):
-
-        user = self.popup_menu.get_user()
-
-        for i in self.usersmodel:
-            if i[2] == user:
-                self.usersmodel.set_value(i.iter, 7, widget.get_active())
-                break
-
-        self.save_user_list()
