@@ -38,7 +38,7 @@ from collections import deque
 from pynicotine import slskmessages
 from pynicotine import slskproto
 from pynicotine import transfers
-from pynicotine.config import Config
+from pynicotine.config import config
 from pynicotine.geoip.ip2location import IP2Location
 from pynicotine.logfacility import log
 from pynicotine.networkfilter import NetworkFilter
@@ -50,6 +50,9 @@ from pynicotine.transfers import Statistics
 from pynicotine.utils import unescape
 
 
+global config
+
+
 class UserAddr:
 
     __slots__ = "addr", "status"
@@ -106,7 +109,7 @@ class ConnectToPeerTimeout:
 class NetworkEventProcessor:
     """ This class contains handlers for various messages from the networking thread """
 
-    def __init__(self, ui_callback, network_callback, setstatus, bindip, port, data_dir, config):
+    def __init__(self, ui_callback, network_callback, setstatus, bindip, port):
 
         self.ui_callback = ui_callback
         self.network_callback = network_callback
@@ -116,51 +119,29 @@ class NetworkEventProcessor:
         # Tell threads when we're disconnecting
         self.exit = threading.Event()
 
-        try:
-            self.config = Config(config, data_dir)
-
-        except Exception:
-            corruptfile = ".".join([config, time.strftime("%Y-%m-%d_%H_%M_%S"), "corrupt"])
-
-            import shutil
-            shutil.move(config, corruptfile)
-
-            short_message = _("Your config file is corrupt")
-            long_message = _("We're sorry, but it seems your configuration file is corrupt. Please reconfigure Nicotine+.\n\nWe renamed your old configuration file to\n%(corrupt)s\nIf you open this file with a text editor you might be able to rescue some of your settings.") % {'corrupt': corruptfile}
-            self.ui_callback.show_info_message(short_message, long_message)
-
-            self.config = Config(config, data_dir)
-
         self.bindip = bindip
         self.port = port
 
-        log.set_log_levels(self.config.sections["logging"]["debugmodes"])
-
         self.peerconns = []
         self.watchedusers = set()
         self.ip_requested = set()
         self.users = {}
         self.out_indirect_conn_request_times = {}
+        self.automatic_message_times = {}
 
         script_dir = os.path.dirname(__file__)
         file_path = os.path.join(script_dir, "geoip/ipcountrydb.bin")
         self.geoip = IP2Location(file_path, "SHARED_MEMORY")
 
         self.queue = deque()
-        self.network_filter = NetworkFilter(self, self.config, self.users, self.queue, self.geoip)
-        self.statistics = Statistics(self.config, self.ui_callback)
-        self.shares = Shares(self, self.config, self.queue, self.ui_callback)
+        self.network_filter = NetworkFilter(self, config, self.users, self.queue, self.geoip)
+        self.statistics = Statistics(config, self.ui_callback)
+        self.shares = Shares(self, config, self.queue, self.ui_callback)
+        self.search = Search(self, config, self.queue, self.shares.share_dbs, self.ui_callback)
         self.pluginhandler = None  # Initialized when the GUI is ready
-        self.now_playing = NowPlaying(self.config)
+        self.now_playing = NowPlaying(config)
 
-        # Give the logger information about log folder
-        should_log = self.config.sections["logging"]["debug_file_output"]
-        log_folder = self.config.sections["logging"]["debuglogsdir"]
-        timestamp_format = self.config.sections["logging"]["log_timestamp"]
-
-        log.update_debug_log_options(should_log, log_folder, timestamp_format)
-
-        port_range = self.config.sections["server"]["portrange"]
+        port_range = config.sections["server"]["portrange"]
         self.protothread = slskproto.SlskProtoThread(self.network_callback, self.queue, self.bindip, self.port, port_range, self.network_filter, self)
 
         # UPnP
@@ -183,7 +164,6 @@ class NetworkEventProcessor:
         self.has_parent = False
 
         self.requested_info = {}
-        self.requested_folders = {}
         self.speed = 0
 
         # Callback handlers for messages
@@ -328,7 +308,7 @@ class NetworkEventProcessor:
         msg_type = msg.type
         found_conn = False
 
-        if user != self.config.sections["server"]["login"]:  # We need two connections in our name if we're downloading from ourselves
+        if user != config.sections["server"]["login"]:  # We need two connections in our name if we're downloading from ourselves
             for i in self.peerconns:
                 if i.username == user and i.type != 'F' and i.type == msg_type:
                     i.addr = addr
@@ -416,7 +396,7 @@ class NetworkEventProcessor:
         else:
             message_type = 'P'
 
-        init = slskmessages.PeerInit(None, self.config.sections["server"]["login"], message_type, 0)
+        init = slskmessages.PeerInit(None, config.sections["server"]["login"], message_type, 0)
         addr = None
 
         if user in self.users:
@@ -488,7 +468,7 @@ class NetworkEventProcessor:
 
         init = slskmessages.PeerInit(None, user, msg_type, 0)
 
-        if user != self.config.sections["server"]["login"]:
+        if user != config.sections["server"]["login"]:
             for i in self.peerconns:
                 if i.username == user and i.type != 'F' and i.type == msg_type:
                     """ Only update existing connection if it hasn't been established yet,
@@ -821,7 +801,7 @@ class NetworkEventProcessor:
             if self.transfers is not None:
                 self.transfers.disconnect()
 
-            self.privatechat = self.chatrooms = self.userinfo = self.userbrowse = self.search = self.transfers = self.userlist = None
+            self.privatechat = self.chatrooms = self.userinfo = self.userbrowse = self.transfers = self.userlist = None
             self.ui_callback.conn_close(conn, addr)
             self.pluginhandler.server_disconnect_notification(userchoice)
 
@@ -908,7 +888,7 @@ class NetworkEventProcessor:
         if self.upnp_timer:
             self.upnp_timer.cancel()
 
-        upnp_interval = self.config.sections["server"]["upnp_interval"]
+        upnp_interval = config.sections["server"]["upnp_interval"]
 
         if upnp_interval < 1:
             return
@@ -923,7 +903,7 @@ class NetworkEventProcessor:
     def add_upnp_portmapping(self):
 
         # Test if we want to do a port mapping
-        if not self.config.sections["server"]["upnp"]:
+        if not config.sections["server"]["upnp"]:
             return
 
         # Do the port mapping
@@ -957,7 +937,7 @@ class NetworkEventProcessor:
         self.set_status(_("The server seems to be down or not responding, retrying in %i seconds"), (self.server_timeout_value))
 
     def server_timeout(self):
-        if not self.config.need_config():
+        if not config.need_config():
             self.ui_callback.on_connect()
 
     def transfer_timeout(self, msg):
@@ -971,6 +951,9 @@ class NetworkEventProcessor:
         """ Tell the server we want to be notified of status/stat updates
         for a user """
 
+        if not isinstance(user, str):
+            return
+
         if not force_update and user in self.watchedusers:
             # Already being watched, and we don't need to re-fetch the status/stats
             return
@@ -980,6 +963,18 @@ class NetworkEventProcessor:
         # Get privilege status
         self.queue.append(slskmessages.GetUserStatus(user))
 
+    def send_automatic_message(self, user, message):
+        """ Sends a private message with the prefix 'Automatic Message' to a user.
+        No message is sent if less than five seconds have passed since the last one. """
+
+        send_time = time.time()
+
+        if user in self.automatic_message_times and (send_time - self.automatic_message_times[user]) < 5:
+            return
+
+        self.queue.append(slskmessages.MessageUser(user, "[Automatic Message] " + message))
+        self.automatic_message_times[user] = send_time
+
     def stop_timers(self):
         if self.servertimer is not None:
             self.servertimer.cancel()
@@ -1017,8 +1012,8 @@ class NetworkEventProcessor:
         self.users.clear()
         self.queue.append(
             slskmessages.Login(
-                self.config.sections["server"]["login"],
-                self.config.sections["server"]["passw"],
+                config.sections["server"]["login"],
+                config.sections["server"]["passw"],
 
                 # Soulseek client version; 155, 156, 157
                 # SoulseekQt seems to be using 157
@@ -1104,17 +1099,16 @@ class NetworkEventProcessor:
             thread.start()
 
             self.queue.append(slskmessages.SetStatus((not self.ui_callback.away) + 1))
-            self.watch_user(self.config.sections["server"]["login"])
+            self.watch_user(config.sections["server"]["login"])
 
-            self.search = Search(self, self.config, self.queue, self.shares.share_dbs, self.ui_callback)
-            self.transfers = transfers.Transfers(self.peerconns, self.queue, self, self.users,
+            self.transfers = transfers.Transfers(config, self.peerconns, self.queue, self, self.users,
                                                  self.network_callback, self.ui_callback.notifications, self.pluginhandler)
             self.shares.set_connected(True)
 
             if msg.ip is not None:
                 self.ipaddress = msg.ip
 
-            for i in self.config.sections["server"]["userlist"]:
+            for i in config.sections["server"]["userlist"]:
                 user = i[0]
                 self.watch_user(user)
 
@@ -1122,11 +1116,11 @@ class NetworkEventProcessor:
 
             self.transfers.set_transfer_views(downloads, uploads)
 
-            for thing in self.config.sections["interests"]["likes"]:
+            for thing in config.sections["interests"]["likes"]:
                 if thing and isinstance(thing, str):
                     self.queue.append(slskmessages.AddThingILike(thing))
 
-            for thing in self.config.sections["interests"]["dislikes"]:
+            for thing in config.sections["interests"]["dislikes"]:
                 if thing and isinstance(thing, str):
                     self.queue.append(slskmessages.AddThingIHate(thing))
 
@@ -1140,21 +1134,24 @@ class NetworkEventProcessor:
             children for now. """
             self.queue.append(slskmessages.AcceptChildren(0))
 
-            if self.shares.initiated_shares and \
-                    not self.shares.public_rescanning and not self.shares.buddy_rescanning:
-                self.shares.send_num_shared_folders_files()
+            self.shares.send_num_shared_folders_files()
 
             """ Request a complete room list. A limited room list not including blacklisted rooms and
             rooms with few users is automatically sent when logging in, but subsequent room list
             requests contain all rooms. """
             self.queue.append(slskmessages.RoomList())
 
-            self.queue.append(slskmessages.PrivateRoomToggle(self.config.sections["server"]["private_chatrooms"]))
+            self.queue.append(slskmessages.PrivateRoomToggle(config.sections["server"]["private_chatrooms"]))
             self.pluginhandler.server_connect_notification()
             self.set_status("")
         else:
             self.manualdisconnect = True
-            self.set_status(_("Can not log in, reason: %s"), (msg.reason))
+            self.queue.append(slskmessages.ConnClose(self.active_server_conn))
+
+            title = _("Connection Refused")
+            message = _("Can not log in, reason: %s") % msg.reason
+            self.ui_callback.show_info_message(title, message)
+            log.add(message)
 
     def add_user(self, msg):
         """ Server code: 5 """
@@ -1285,7 +1282,7 @@ class NetworkEventProcessor:
         if log_contents:
             log.add_msg_contents(msg)
 
-        if msg.user == self.config.sections["server"]["login"]:
+        if msg.user == config.sections["server"]["login"]:
             self.speed = msg.avgspeed
 
         if self.interests is not None:
@@ -1414,7 +1411,7 @@ class NetworkEventProcessor:
         """ This message embeds a distributed message. We unpack the distributed message and
         process it. """
 
-        log.add_msg_contents(msg)
+        # Verbose: log.add_msg_contents(msg)
 
         if msg.distrib_code in self.protothread.distribclasses:
             distrib_class = self.protothread.distribclasses[msg.distrib_code]
@@ -1553,8 +1550,8 @@ class NetworkEventProcessor:
         log.add_msg_contents(msg)
 
         password = msg.password
-        self.config.sections["server"]["passw"] = password
-        self.config.write_configuration()
+        config.sections["server"]["passw"] = password
+        config.write_configuration()
 
         self.ui_callback.show_info_message(_("Your password has been changed"), _("Password is %s") % password)
 
@@ -1634,7 +1631,7 @@ class NetworkEventProcessor:
             return
 
         # Check address is spoofed, if possible
-        if user == self.config.sections["server"]["login"]:
+        if user == config.sections["server"]["login"]:
             if ip is not None and port is not None:
                 log.add(
                     _("%(user)s is making a BrowseShares request, blocking possible spoofing attempt from IP %(ip)s port %(port)s"), {
@@ -1659,26 +1656,22 @@ class NetworkEventProcessor:
         ip, port = msg.conn.addr
         checkuser, reason = self.network_filter.check_user(user, ip)
 
-        if checkuser == 1 and self.shares.initiated_shares:
+        if not checkuser:
+            self.send_automatic_message(user, reason)
+
+        m = None
+
+        if checkuser == 1:
             # Send Normal Shares
-            if self.shares.newnormalshares:
-                self.shares.create_compressed_shares_message("normal")
-                self.shares.compress_shares("normal")
-                self.shares.newnormalshares = False
-            m = self.shares.compressed_shares_normal
+            m = self.shares.get_compressed_shares_message("normal")
 
-        elif checkuser == 2 and self.shares.initiated_shares:
+        elif checkuser == 2:
             # Send Buddy Shares
-            if self.shares.newbuddyshares:
-                self.shares.create_compressed_shares_message("buddy")
-                self.shares.compress_shares("buddy")
-                self.shares.newbuddyshares = False
-            m = self.shares.compressed_shares_buddy
+            m = self.shares.get_compressed_shares_message("buddy")
 
-        else:
+        if not m:
             # Nyah, Nyah
             m = slskmessages.SharedFileList(conn, {})
-            m.make_network_message(nozlib=0)
 
         m.conn = conn
         self.queue.append(m)
@@ -1690,7 +1683,7 @@ class NetworkEventProcessor:
 
         for i in self.peerconns:
             if i.conn is conn and self.userbrowse is not None:
-                if i.username != self.config.sections["server"]["login"]:
+                if i.username != config.sections["server"]["login"]:
                     indeterminate_progress = change_page = False
                     self.userbrowse.show_user(i.username, None, msg, indeterminate_progress, change_page)
                     break
@@ -1758,16 +1751,15 @@ class NetworkEventProcessor:
 
         request_time = time.time()
 
-        if user in self.requested_info:
-            if not request_time > 10 + self.requested_info[user]:
-                # Ignoring request, because it's 10 or less seconds since the
-                # last one by this user
-                return
+        if user in self.requested_info and not request_time > 10 + self.requested_info[user]:
+            # Ignoring request, because it's 10 or less seconds since the
+            # last one by this user
+            return
 
         self.requested_info[user] = request_time
 
         # Check address is spoofed, if possible
-        if user == self.config.sections["server"]["login"]:
+        if user == config.sections["server"]["login"]:
 
             if ip is not None and port is not None:
                 log.add(
@@ -1795,7 +1787,7 @@ class NetworkEventProcessor:
             return
 
         try:
-            userpic = self.config.sections["userinfo"]["pic"]
+            userpic = config.sections["userinfo"]["pic"]
 
             with open(userpic, 'rb') as f:
                 pic = f.read()
@@ -1803,15 +1795,15 @@ class NetworkEventProcessor:
         except Exception:
             pic = None
 
-        descr = unescape(self.config.sections["userinfo"]["descr"])
+        descr = unescape(config.sections["userinfo"]["descr"])
 
         if self.transfers is not None:
             totalupl = self.transfers.get_total_uploads_allowed()
             queuesize = self.transfers.get_upload_queue_size()
             slotsavail = self.transfers.allow_new_uploads()
 
-            if self.config.sections["transfers"]["remotedownloads"]:
-                uploadallowed = self.config.sections["transfers"]["uploadallowed"]
+            if config.sections["transfers"]["remotedownloads"]:
+                uploadallowed = config.sections["transfers"]["uploadallowed"]
             else:
                 uploadallowed = 0
 
@@ -1833,7 +1825,7 @@ class NetworkEventProcessor:
         for i in self.peerconns:
             if i.conn is conn and self.userinfo is not None:
                 # probably impossible to do this
-                if i.username != self.config.sections["server"]["login"]:
+                if i.username != config.sections["server"]["login"]:
                     indeterminate_progress = change_page = False
                     self.userinfo.show_user(i.username, None, msg, indeterminate_progress, change_page)
                     break
@@ -1886,17 +1878,16 @@ class NetworkEventProcessor:
             return
 
         if not checkuser:
-            self.queue.append(slskmessages.MessageUser(username, "[Automatic Message] " + reason))
-            return
+            self.send_automatic_message(username, reason)
 
-        if not self.shares.initiated_shares:
-            return
+        normalshares = self.shares.share_dbs.get("streams")
+        buddyshares = self.shares.share_dbs.get("buddystreams")
 
-        if checkuser == 1 and not self.shares.public_rescanning:
-            shares = self.shares.share_dbs["streams"]
+        if checkuser == 1 and normalshares is not None:
+            shares = normalshares
 
-        elif checkuser == 2 and not self.shares.buddy_rescanning:
-            shares = self.shares.share_dbs["buddystreams"]
+        elif checkuser == 2 and buddyshares is not None:
+            shares = buddyshares
 
         else:
             shares = {}
@@ -2018,7 +2009,7 @@ class NetworkEventProcessor:
     def distrib_search(self, msg):
         """ Distrib code: 3 """
 
-        log.add_msg_contents(msg)
+        # Verbose: log.add_msg_contents(msg)
 
         if self.search is not None:
             self.search.process_search_request(msg.searchterm, msg.user, msg.searchid, direct=False)
@@ -2040,7 +2031,7 @@ class NetworkEventProcessor:
         log.add_conn("We have no parent, requesting a new one")
 
         self.queue.append(slskmessages.HaveNoParent(1))
-        self.queue.append(slskmessages.BranchRoot(self.config.sections["server"]["login"]))
+        self.queue.append(slskmessages.BranchRoot(config.sections["server"]["login"]))
         self.queue.append(slskmessages.BranchLevel(0))
 
     def distrib_branch_level(self, msg):
