@@ -20,8 +20,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import importlib
-import multiprocessing
+import importlib.util
 import os
 import pickle
 import shelve
@@ -32,8 +31,9 @@ import threading
 import time
 
 from pynicotine import slskmessages
+from pynicotine.i18n import apply_translation
 from pynicotine.logfacility import log
-from pynicotine.utils import apply_translation
+from pynicotine.utils import rename_process
 
 """ Check if there's an appropriate (performant) database type for shelves """
 
@@ -61,14 +61,13 @@ else:
     sys.exit()
 
 
-class Scanner(multiprocessing.Process):
+class Scanner:
     """ Separate process responsible for building shares. It handles scanning of
     folders and files, as well as building databases and writing them to disk. """
 
     def __init__(self, config, queue, shared_folders, sharestype="normal", rebuild=False):
 
         from pynicotine.metadata.tinytag import TinyTag
-        multiprocessing.Process.__init__(self)
 
         self.config = config
         self.queue = queue
@@ -82,6 +81,7 @@ class Scanner(multiprocessing.Process):
     def run(self):
 
         try:
+            rename_process(b'nicotine-scan')
             apply_translation()
 
             if self.sharestype == "normal":
@@ -498,7 +498,10 @@ class Shares:
         self.share_dbs = {}
         self.public_rescanning = False
         self.buddy_rescanning = False
-        self.initiated_shares = False
+        self.newbuddyshares = False
+        self.newnormalshares = False
+        self.compressed_shares_normal = slskmessages.SharedFileList(None, None)
+        self.compressed_shares_buddy = slskmessages.SharedFileList(None, None)
 
         self.convert_shares()
         self.public_share_dbs = [
@@ -525,8 +528,6 @@ class Shares:
         else:
             self.init_shares()
 
-        self.newbuddyshares = self.newnormalshares = False
-
     def set_connected(self, connected):
         self.connected = connected
 
@@ -589,12 +590,6 @@ class Shares:
 
     def init_shares(self):
 
-        self.load_shares(self.share_dbs, self.public_share_dbs)
-        self.load_shares(self.share_dbs, self.buddy_share_dbs)
-
-        self.create_compressed_shares_message("normal")
-        self.create_compressed_shares_message("buddy")
-
         rescan_startup = self.config.sections["transfers"]["rescanonstartup"]
 
         # Rescan public shares if necessary
@@ -602,6 +597,8 @@ class Shares:
             if rescan_startup and not self.public_rescanning:
                 self.rescan_public_shares()
             else:
+                self.load_shares(self.share_dbs, self.public_share_dbs)
+                self.create_compressed_shares_message("normal")
                 self.compress_shares("normal")
 
         # Rescan buddy shares if necessary
@@ -609,10 +606,10 @@ class Shares:
             if rescan_startup and not self.buddy_rescanning:
                 self.rescan_buddy_shares()
             else:
+                self.load_shares(self.share_dbs, self.buddy_share_dbs)
+                self.create_compressed_shares_message("buddy")
                 self.compress_shares("buddy")
 
-        self.initiated_shares = True
-
     def convert_shares(self):
         """ Convert fs-based shared to virtual shared (pre 1.4.0) """
 
@@ -655,114 +652,160 @@ class Shares:
             if not reset_shares:
                 cls.load_shares(shares, dbs, reset_shares=True)
 
+    def file_is_shared(self, user, virtualfilename, realfilename):
+
+        log.add_transfer("Checking if file %(virtual_name)s with real path %(path)s is shared", {
+            "virtual_name": virtualfilename,
+            "path": realfilename
+        })
+
+        if not os.access(realfilename, os.R_OK):
+            log.add_transfer("Can't access file %(virtual_name)s with real path %(path)s, not sharing", {
+                "virtual_name": virtualfilename,
+                "path": realfilename
+            })
+            return False
+
+        folder, sep, file = virtualfilename.rpartition('\\')
+        shared = self.share_dbs.get("files")
+        bshared = self.share_dbs.get("buddyfiles")
+
+        if self.config.sections["transfers"]["enablebuddyshares"] and bshared is not None:
+            if user in (i[0] for i in self.config.sections["server"]["userlist"]):
+                for i in bshared.get(str(folder), ''):
+                    if file == i[0]:
+                        return True
+
+        if shared is not None:
+            for i in shared.get(str(folder), ''):
+                if file == i[0]:
+                    return True
+
+        log.add_transfer("Failed to share file %(virtual_name)s with real path %(path)s, since it wasn't found", {
+            "virtual_name": virtualfilename,
+            "path": realfilename
+        })
+        return False
+
     def add_file_to_shared(self, name):
         """ Add a file to the normal shares database """
 
-        config = self.config.sections
-        if not config["transfers"]["sharedownloaddir"]:
+        if not self.config.sections["transfers"]["sharedownloaddir"]:
             return
 
-        if not self.initiated_shares or self.public_rescanning:
+        shared = self.share_dbs.get("files")
+
+        if shared is None:
             return
 
-        shared = self.share_dbs["files"]
-        sharedstreams = self.share_dbs["streams"]
-        wordindex = self.share_dbs["wordindex"]
-        fileindex = self.share_dbs["fileindex"]
-
-        shareddirs = [path for _name, path in config["transfers"]["shared"]]
-        shareddirs.append(config["transfers"]["downloaddir"])
-
-        sharedmtimes = self.share_dbs["mtimes"]
+        shareddirs = [path for _name, path in self.config.sections["transfers"]["shared"]]
+        shareddirs.append(self.config.sections["transfers"]["downloaddir"])
 
         rdir = str(os.path.expanduser(os.path.dirname(name)))
         vdir = self.real2virtual(rdir)
         file = str(os.path.basename(name))
 
-        shared[vdir] = shared.get(vdir, [])
+        if not shared.get(vdir):
+            shared[vdir] = []
 
         if file not in (i[0] for i in shared[vdir]):
             from pynicotine.metadata.tinytag import TinyTag
             fileinfo = Scanner.get_file_info(file, name, TinyTag())
             shared[vdir] += [fileinfo]
 
-            sharedstreams[vdir] = Scanner.get_dir_stream(shared[vdir])
+            self.share_dbs["streams"][vdir] = Scanner.get_dir_stream(shared[vdir])
+
+            fileindex = self.share_dbs["fileindex"]
 
             try:
                 index = len(fileindex)
             except TypeError:
                 index = len(list(fileindex))
 
-            Scanner.add_file_to_index(index, file, vdir, fileinfo, wordindex, fileindex, self.translatepunctuation)
+            Scanner.add_file_to_index(index, file, vdir, fileinfo, self.share_dbs["wordindex"], fileindex, self.translatepunctuation)
 
-            sharedmtimes[vdir] = os.path.getmtime(rdir)
+            self.share_dbs["mtimes"][vdir] = os.path.getmtime(rdir)
             self.newnormalshares = True
 
-        if config["transfers"]["enablebuddyshares"]:
-            self.add_file_to_buddy_shared(name)
-
     def add_file_to_buddy_shared(self, name):
         """ Add a file to the buddy shares database """
 
-        config = self.config.sections
-        if not config["transfers"]["sharedownloaddir"]:
+        if not self.config.sections["transfers"]["sharedownloaddir"]:
             return
 
-        if not self.initiated_shares or self.buddy_rescanning:
+        if not self.config.sections["transfers"]["enablebuddyshares"]:
             return
 
-        bshared = self.share_dbs["buddyfiles"]
-        bsharedstreams = self.share_dbs["buddystreams"]
-        bwordindex = self.share_dbs["buddywordindex"]
-        bfileindex = self.share_dbs["buddyfileindex"]
+        bshared = self.share_dbs.get("buddyfiles")
 
-        bshareddirs = [path for _name, path in config["transfers"]["shared"]]
-        bshareddirs += [path for _name, path in config["transfers"]["buddyshared"]]
-        bshareddirs.append(config["transfers"]["downloaddir"])
+        if bshared is None:
+            return
 
-        bsharedmtimes = self.share_dbs["buddymtimes"]
+        bshareddirs = [path for _name, path in self.config.sections["transfers"]["shared"]]
+        bshareddirs += [path for _name, path in self.config.sections["transfers"]["buddyshared"]]
+        bshareddirs.append(self.config.sections["transfers"]["downloaddir"])
 
         rdir = str(os.path.expanduser(os.path.dirname(name)))
         vdir = self.real2virtual(rdir)
         file = str(os.path.basename(name))
 
-        bshared[vdir] = bshared.get(vdir, [])
+        if not bshared.get(vdir):
+            bshared[vdir] = []
 
         if file not in (i[0] for i in bshared[vdir]):
             from pynicotine.metadata.tinytag import TinyTag
             fileinfo = Scanner.get_file_info(file, name, TinyTag())
             bshared[vdir] += [fileinfo]
 
-            bsharedstreams[vdir] = Scanner.get_dir_stream(bshared[vdir])
+            self.share_dbs["buddystreams"][vdir] = Scanner.get_dir_stream(bshared[vdir])
+
+            bfileindex = self.share_dbs["buddyfileindex"]
 
             try:
                 index = len(bfileindex)
             except TypeError:
                 index = len(list(bfileindex))
 
-            Scanner.add_file_to_index(index, file, vdir, fileinfo, bwordindex, bfileindex, self.translatepunctuation)
+            Scanner.add_file_to_index(
+                index, file, vdir, fileinfo,
+                self.share_dbs["buddywordindex"], bfileindex, self.translatepunctuation
+            )
 
-            bsharedmtimes[vdir] = os.path.getmtime(rdir)
+            self.share_dbs["buddymtimes"][vdir] = os.path.getmtime(rdir)
             self.newbuddyshares = True
 
     def create_compressed_shares_message(self, sharestype):
-
         """ Create a message that will later contain a compressed list of our shares """
 
         if sharestype == "normal":
             self.compressed_shares_normal = slskmessages.SharedFileList(
                 None,
-                self.share_dbs["streams"]
+                self.share_dbs.get("streams")
             )
 
         elif sharestype == "buddy":
             self.compressed_shares_buddy = slskmessages.SharedFileList(
                 None,
-                self.share_dbs["buddystreams"]
+                self.share_dbs.get("buddystreams")
             )
 
-    def compress_shares(self, sharestype):
+    def get_compressed_shares_message(self, sharestype):
+        """ Returns the compressed shares message. Creates a new one if necessary, e.g.
+        if an individual file was added to our shares. """
 
+        if sharestype == "normal" and self.newnormalshares or \
+                sharestype == "buddy" and self.newbuddyshares:
+            self.create_compressed_shares_message(sharestype)
+
+        if sharestype == "normal":
+            self.newnormalshares = False
+            return self.compressed_shares_normal
+
+        if sharestype == "buddy":
+            self.newbuddyshares = False
+            return self.compressed_shares_buddy
+
+    def compress_shares(self, sharestype):
         """ Begin compressing the shares list. This compressed list will be used to
         quickly send our file list to users. """
 
@@ -790,33 +833,35 @@ class Shares:
             ]
 
         for db in dbs:
-            try:
+            db_file = self.share_dbs.get(db)
+
+            if db_file is not None:
                 self.share_dbs[db].close()
-            except AttributeError:
-                continue
+                del self.share_dbs[db]
 
     def send_num_shared_folders_files(self):
         """ Send number publicly shared files to the server. """
 
-        config = self.config.sections
-
-        if config["transfers"]["enablebuddyshares"] and config["transfers"]["friendsonly"]:
+        if self.config.sections["transfers"]["friendsonly"]:
             # No public shares
             files = folders = 0
             self.queue.append(slskmessages.SharedFoldersFiles(files, folders))
             return
 
-        shared_db = "files"
-        index_db = "fileindex"
-
         try:
+            shared = self.share_dbs.get("files")
+            index = self.share_dbs.get("fileindex")
+
+            if shared is None or index is None:
+                return
+
             try:
-                sharedfolders = len(self.share_dbs[shared_db])
-                sharedfiles = len(self.share_dbs[index_db])
+                sharedfolders = len(shared)
+                sharedfiles = len(index)
 
             except TypeError:
-                sharedfolders = len(list(self.share_dbs[shared_db]))
-                sharedfiles = len(list(self.share_dbs[index_db]))
+                sharedfolders = len(list(shared))
+                sharedfiles = len(list(index))
 
             self.queue.append(slskmessages.SharedFoldersFiles(sharedfolders, sharedfiles))
 
@@ -827,6 +872,10 @@ class Shares:
 
     def build_scanner_process(self, shared_folders=None, sharestype="normal", rebuild=False):
 
+        import multiprocessing
+
+        multiprocessing.set_start_method("spawn", force=True)
+
         scanner_queue = multiprocessing.Queue()
         scanner = Scanner(
             self.config,
@@ -835,6 +884,7 @@ class Shares:
             sharestype,
             rebuild
         )
+        scanner = multiprocessing.Process(target=scanner.run)
         scanner.daemon = True
         return scanner, scanner_queue
 
@@ -930,10 +980,8 @@ class Shares:
 
         # Scanning done, load shares in the main process again
         if sharestype == "normal":
-            self.public_rescanning = False
             self.load_shares(self.share_dbs, self.public_share_dbs)
         else:
-            self.buddy_rescanning = False
             self.load_shares(self.share_dbs, self.buddy_share_dbs)
 
         self.create_compressed_shares_message(sharestype)
@@ -951,5 +999,10 @@ class Shares:
             else:
                 log.add(_("Finished rescanning buddy shares"))
 
+        if sharestype == "normal":
+            self.public_rescanning = False
+        else:
+            self.buddy_rescanning = False
+
         if self.ui_callback:
             self.ui_callback.hide_scan_progress(sharestype)
