@@ -22,7 +22,6 @@ import socket
 import struct
 import zlib
 
-from hashlib import md5
 from itertools import count
 from itertools import islice
 
@@ -340,6 +339,8 @@ class Login(ServerMessage):
         self.ip = None
 
     def make_network_message(self):
+        from hashlib import md5
+
         msg = bytearray()
         msg.extend(self.pack_object(self.username))
         msg.extend(self.pack_object(self.passwd))
@@ -2063,23 +2064,23 @@ class SharedFileList(PeerMessage):
 
         msg = bytearray()
 
-        try:
+        if self.list is None:
+            # DB is closed
+            msg.extend(self.pack_object(0))
+
+        else:
             try:
                 msg.extend(self.pack_object(len(self.list)))
 
             except TypeError:
                 msg.extend(self.pack_object(len(list(self.list))))
 
-        except ValueError:
-            # DB is closed
-            return None
-
-        for key in self.list:
-            try:
-                msg.extend(self.pack_object(key.replace('/', '\\')))
-                msg.extend(self.list[key])
-            except KeyError:
-                pass
+            for key in self.list:
+                try:
+                    msg.extend(self.pack_object(key.replace('/', '\\')))
+                    msg.extend(self.list[key])
+                except KeyError:
+                    pass
 
         if not nozlib:
             self.built = zlib.compress(msg)
@@ -2417,9 +2418,13 @@ class FolderContentsResponse(PeerMessage):
 
 class TransferRequest(PeerMessage):
     """ Peer code: 40 """
-    """ This message is sent when attempting to initiate an upload. It was formely used
-    to send a download request as well, but Nicotine+, Museek+ and the official clients
-    use QueueUpload for this purpose today. """
+    """ This message is sent by a peer once they are ready to start uploading a file.
+    A TransferResponse message is expected from the recipient, either allowing or
+    rejecting the upload attempt.
+
+    This message was formely used to send a download request (direction 0) as well,
+    but Nicotine+, Museek+ and the official clients use the QueueUpload message for
+    this purpose today. """
 
     def __init__(self, conn, direction=None, req=None, file=None, filesize=None, realfile=None, legacy_client=False):
         self.conn = conn
