@@ -883,6 +883,11 @@ class SlskProtoThread(threading.Thread):
 
     def close_connection(self, connection_list, connection):
         try:
+            if connection is self._server_socket:
+                # Disconnected from server, clean up connections and queue
+                self.server_disconnect()
+                return
+
             connection.close()
             del connection_list[connection]
 
@@ -992,10 +997,6 @@ class SlskProtoThread(threading.Thread):
                     if msg_obj.callback:
                         self._ui_callback([ConnClose(conn, conns[conn].addr)])
 
-                    if conn == server_socket:
-                        # Disconnected from server, clean up connections and queue
-                        self.server_disconnect()
-
                     self.close_connection(conns, conn)
 
                 elif msg_obj.__class__ is OutConn:
@@ -1265,8 +1266,12 @@ class SlskProtoThread(threading.Thread):
             curtime = time.time()
 
             for connection_in_progress in connsinprogress.copy():
+                conn_obj = connsinprogress.get(connection_in_progress)
+
+                if not conn_obj:
+                    # Connection was removed, possibly disconnecting from the server
+                    continue
 
-                conn_obj = connsinprogress[connection_in_progress]
                 msg_obj = conn_obj.msg_obj
 
                 if (curtime - conn_obj.lastactive) > self.IN_PROGRESS_STALE_AFTER:
@@ -1278,7 +1283,9 @@ class SlskProtoThread(threading.Thread):
                 try:
                     if connection_in_progress in input_list:
                         # Check if the socket has any data for us
+                        connection_in_progress.setblocking(0)
                         connection_in_progress.recv(1, socket.MSG_PEEK)
+                        connection_in_progress.setblocking(1)
 
                 except socket.error as err:
 
@@ -1304,7 +1311,11 @@ class SlskProtoThread(threading.Thread):
             curtime = time.time()
 
             for connection in conns.copy():
-                conn_obj = conns[connection]
+                conn_obj = conns.get(connection)
+
+                if not conn_obj:
+                    # Connection was removed, possibly disconnecting from the server
+                    continue
 
                 if connection in output_list:
                     # Write Output
@@ -1394,11 +1405,12 @@ class SlskProtoThread(threading.Thread):
 
         for i in self._conns.copy():
             i.close()
-            del self._conns[i]
 
         for i in self._connsinprogress.copy():
             i.close()
-            del self._connsinprogress[i]
+
+        self._conns.clear()
+        self._connsinprogress.clear()
 
         while self._queue:
             self._queue.popleft()
