@@ -26,15 +26,14 @@ This module contains utility functions.
 """
 
 import errno
-import gettext
-import locale
 import os
 import pickle
 import sys
 
+from pynicotine.config import config
 from pynicotine.logfacility import log
 
-version = "3.0.4"
+version = "3.0.6"
 
 win32 = sys.platform.startswith("win")
 
@@ -43,6 +42,38 @@ illegalfilechars = illegalpathchars + ["\\", "/"]
 replacementchar = '_'
 
 
+def rename_process(new_name, debug=False):
+
+    errors = []
+
+    # Renaming ourselves for pkill et al.
+    try:
+        import ctypes
+        # GNU/Linux style
+        libc = ctypes.CDLL(None)
+        libc.prctl(15, new_name, 0, 0, 0)
+
+    except Exception as e:
+        errors.append(e)
+        errors.append("Failed GNU/Linux style")
+
+        try:
+            import ctypes
+            # BSD style
+            libc = ctypes.CDLL(None)
+            libc.setproctitle(new_name)
+
+        except Exception as e:
+            errors.append(e)
+            errors.append("Failed BSD style")
+
+    if debug and errors:
+        msg = ["Errors occurred while trying to change process name:"]
+        for i in errors:
+            msg.append("%s" % (i,))
+        print('\n'.join(msg))
+
+
 def clean_file(filename):
 
     for char in illegalfilechars:
@@ -126,40 +157,6 @@ def make_version(version):
     return (major << 24) + (minor << 16) + (patch << 8) + stable
 
 
-def get_user_directories():
-    """Returns a tuple:
-    - the config directory
-    - the data directory"""
-
-    if win32:
-        try:
-            data_dir = os.path.join(os.environ['APPDATA'], 'nicotine')
-        except KeyError:
-            data_dir, x = os.path.split(sys.argv[0])
-
-        config_dir = os.path.join(data_dir, "config")
-        return config_dir, data_dir
-
-    home = os.path.expanduser("~")
-
-    legacy_dir = os.path.join(home, '.nicotine')
-
-    if os.path.isdir(legacy_dir):
-        return legacy_dir, legacy_dir
-
-    def xdg_path(xdg, default):
-        path = os.environ.get(xdg)
-
-        path = path.split(':')[0] if path else default
-
-        return os.path.join(path, 'nicotine')
-
-    config_dir = xdg_path('XDG_CONFIG_HOME', os.path.join(home, '.config'))
-    data_dir = xdg_path('XDG_DATA_HOME', os.path.join(home, '.local', 'share'))
-
-    return config_dir, data_dir
-
-
 def get_result_bitrate_length(filesize, attributes):
     """ Used to get the audio bitrate and length of search results and
     user browse files """
@@ -260,51 +257,63 @@ def get_result_bitrate_length(filesize, attributes):
     return h_bitrate, bitrate, h_length, length
 
 
-def apply_translation():
-    """Function dealing with translations and locales.
+size_suffixes = ['B', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB']
 
-    We try to autodetect the language and fix the locale.
 
-    If something goes wrong we fall back to no translation.
-
-    This function also try to find translation files in the project path first:
-    $(PROJECT_PATH)/mo/$(LANG)/LC_MESSAGES/nicotine.mo
-
-    If no translations are found we fall back to the system path for locates:
-    GNU/Linux: /usr/share/locale/$(LANG)/LC_MESSAGES
-    Windows: %PYTHONHOME%\\share\\locale\\$(LANG)\\LC_MESSAGES
-
-    Note: To the best of my knowledge when we are in a python venv
-    falling back to the system path does not work."""
-
-    # Locales handling: We let the system handle the locales
+def human_size(filesize):
     try:
-        locale.setlocale(locale.LC_ALL, '')
+        step_unit = 1024.0
 
-    except Exception as e:
-        print("Error while attempting to set locale: %s" % e)
+        for i in size_suffixes:
+            if filesize < step_unit:
+                return "%3.1f %s" % (filesize, i)
 
-    # package name for gettext
-    package = 'nicotine'
+            filesize /= step_unit
+    except TypeError:
+        return filesize
 
-    # Local path where to find translation (mo) files
-    local_mo_path = 'mo'
 
-    # Gettext handling
-    if gettext.find(package, localedir=local_mo_path) is None:
+speed_suffixes = ['B/s', 'KiB/s', 'MiB/s', 'GiB/s', 'TiB/s', 'PiB/s', 'EiB/s', 'ZiB/s', 'YiB/s']
 
-        # Locales are not in the current dir
-        # We let gettext handle the situation: if if found them in the system dir
-        # the app will be trnaslated, if not it will be untranslated.
-        gettext.install(package)
 
+def human_speed(filesize):
+    try:
+        step_unit = 1024.0
+
+        for i in speed_suffixes:
+            if filesize < step_unit:
+                return "%3.1f %s" % (filesize, i)
+
+            filesize /= step_unit
+    except TypeError:
+        return filesize
+
+
+def humanize(number):
+
+    fashion = config.sections["ui"]["decimalsep"]
+
+    if fashion == "" or fashion == "<None>":
+        return str(number)
+
+    elif fashion == "<space>":
+        fashion = " "
+
+    number = str(number)
+
+    if number[0] == "-":
+        neg = "-"
+        number = number[1:]
     else:
+        neg = ""
 
-        # Locales are in the current dir: install them
-        gettext.bindtextdomain(package, local_mo_path)
-        gettext.install(package, local_mo_path)
+    ret = ""
 
-    gettext.textdomain(package)
+    while number[-3:]:
+        part, number = number[-3:], number[:-3]
+        ret = "%s%s%s" % (part, fashion, ret)
+
+    return neg + ret[:-1]
 
 
 def unescape(string):
@@ -510,6 +519,169 @@ class RestrictedUnpickler(pickle.Unpickler):
                                      (module, name))
 
 
+""" Command Aliases """
+
+
+def add_alias(rest):
+
+    aliases = config.sections["server"]["command_aliases"]
+
+    if rest:
+        args = rest.split(" ", 1)
+
+        if len(args) == 2:
+            if args[0] in ("alias", "unalias"):
+                return "I will not alias that!\n"
+
+            aliases[args[0]] = args[1]
+
+        if args[0] in aliases:
+            return "Alias %s: %s\n" % (args[0], aliases[args[0]])
+        else:
+            return _("No such alias (%s)") % rest + "\n"
+
+    else:
+        m = "\n" + _("Aliases:") + "\n"
+
+        for (key, value) in aliases.items():
+            m = m + "%s: %s\n" % (key, value)
+
+        return m + "\n"
+
+
+def unalias(rest):
+
+    aliases = config.sections["server"]["command_aliases"]
+
+    if rest and rest in aliases:
+        x = aliases[rest]
+        del aliases[rest]
+
+        return _("Removed alias %(alias)s: %(action)s\n") % {'alias': rest, 'action': x}
+
+    else:
+        return _("No such alias (%(alias)s)\n") % {'alias': rest}
+
+
+def is_alias(cmd):
+
+    if not cmd:
+        return False
+
+    if cmd[0] != "/":
+        return False
+
+    cmd = cmd[1:].split(" ")
+
+    if cmd[0] in config.sections["server"]["command_aliases"]:
+        return True
+
+    return False
+
+
+def expand_alias(cmd):
+    output = _expand_alias(config.sections["server"]["command_aliases"], cmd)
+    return output
+
+
+def _expand_alias(aliases, cmd):
+
+    def getpart(line):
+        if line[0] != "(":
+            return ""
+        ix = 1
+        ret = ""
+        level = 0
+        while ix < len(line):
+            if line[ix] == "(":
+                level = level + 1
+            if line[ix] == ")":
+                if level == 0:
+                    return ret
+                else:
+                    level = level - 1
+            ret = ret + line[ix]
+            ix = ix + 1
+        return ""
+
+    if not is_alias(cmd):
+        return None
+    try:
+        cmd = cmd[1:].split(" ")
+        alias = aliases[cmd[0]]
+        ret = ""
+        i = 0
+        while i < len(alias):
+            if alias[i:i + 2] == "$(":
+                arg = getpart(alias[i + 1:])
+                if not arg:
+                    ret = ret + "$"
+                    i = i + 1
+                    continue
+                i = i + len(arg) + 3
+                args = arg.split("=", 1)
+                if len(args) > 1:
+                    default = args[1]
+                else:
+                    default = ""
+                args = args[0].split(":")
+                if len(args) == 1:
+                    first = last = int(args[0])
+                else:
+                    if args[0]:
+                        first = int(args[0])
+                    else:
+                        first = 1
+                    if args[1]:
+                        last = int(args[1])
+                    else:
+                        last = len(cmd)
+                v = " ".join(cmd[first:last + 1])
+                if not v:
+                    v = default
+                ret = ret + v
+            else:
+                ret = ret + alias[i]
+                i = i + 1
+        return ret
+    except Exception as error:
+        log.add_warning("%s", error)
+
+    return ""
+
+
+""" Chat Completion """
+
+
+def get_completion_list(commands, rooms):
+
+    config_words = config.sections["words"]
+
+    if not config_words["tab"]:
+        return []
+
+    completion_list = [config.sections["server"]["login"], "nicotine"]
+
+    if config_words["roomnames"]:
+        completion_list += rooms
+
+    if config_words["buddies"]:
+        for i in config.sections["server"]["userlist"]:
+            user = i[0]
+            if isinstance(user, str):
+                completion_list.append(user)
+
+    if config_words["aliases"]:
+        for k in config.sections["server"]["command_aliases"].keys():
+            if isinstance(k, str):
+                completion_list.append("/" + k)
+
+    if config_words["commands"]:
+        completion_list += commands
+
+    return completion_list
+
+
 """ Debugging """
 
 
